# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from typing import TYPE_CHECKING, Dict, Any, Optional, Sequence, Union, cast
from datetime import datetime

from ._shared.models import (
    CommunicationIdentifier,
    CommunicationUserIdentifier,
    PhoneNumberIdentifier,
    MicrosoftTeamsUserIdentifier,
    MicrosoftTeamsAppIdentifier,
    TeamsExtensionUserIdentifier,
    UnknownIdentifier,
    CommunicationIdentifierKind,
)
from ._generated.models import (
    CommunicationIdentifierModel,
    CommunicationUserIdentifierModel,
    MicrosoftTeamsUserIdentifierModel,
    PhoneNumberIdentifierModel,
    TeamsExtensionUserIdentifierModel,
    CallLocator,
    ExternalStorage,
    RecordingStorageKind,
    MicrosoftTeamsAppIdentifierModel,
)

if TYPE_CHECKING:
    from ._models import (
        ServerCallLocator,
        GroupCallLocator,
        RoomCallLocator,
        AzureBlobContainerRecordingStorage,
        AzureCommunicationsRecordingStorage,
    )


def build_external_storage(
    recording_storage: Optional[
        Union["AzureCommunicationsRecordingStorage", "AzureBlobContainerRecordingStorage"]
    ] = None
) -> Optional[ExternalStorage]:
    request: Optional[ExternalStorage] = None
    if recording_storage:
        if recording_storage.kind == RecordingStorageKind.AZURE_BLOB_STORAGE:
            if not recording_storage.container_url:
                raise ValueError(
                    "Please provide container_url"
                    "when you set the recording_storage as AzureBlobContainerRecordingStorage"
                )
            request = ExternalStorage(
                recording_storage_kind=recording_storage.kind,
                recording_destination_container_url=recording_storage.container_url,
            )
    return request


def build_call_locator(
    call_locator: Optional[Union["ServerCallLocator", "GroupCallLocator", "RoomCallLocator"]],
    server_call_id: Optional[str],
    group_call_id: Optional[str],
    room_id: Optional[str],
    args: Optional[Sequence[Union["ServerCallLocator", "GroupCallLocator", "RoomCallLocator"]]] = None,
) -> Optional[CallLocator]:
    """Build the generated callLocator object from args in kwargs with support for legacy models.

    :param args: Any positional parameters provided. This may include the legacy model. The new method signature
     does not support positional params, so if there's anything here, it's the old model.
    :type args: list[ServerCallLocator or GroupCallLocator or RoomCallLocator]
    :param call_locator: If the legacy call_locator was provided via keyword arg.
    :type call_locator: ServerCallLocator or GroupCallLocator or RoomCallLocator or None
    :param server_call_id: If the new server_call_id was provided via keyword arg.
    :type server_call_id: str or None
    :param group_call_id: If the new group_call_id was provided via keyword arg.
    :type group_call_id: str or None
    :param room_id: If the new room_id was provided via keyword arg.
    :type room_id: str or None
    :return: Generated CallLocator for the request body.
    """
    request: Optional[CallLocator] = None
    if args:
        if len(args) > 1:
            raise TypeError(f"Unexpected positional arguments: {args[1:]}")
        request = args[0]._to_generated()  # pylint:disable=protected-access

    if call_locator:
        if request is not None:
            raise ValueError(
                "Received multiple values for call_locator. "
                "Please provide either 'group_call_id' or 'server_call_id'."
            )
        request = call_locator._to_generated()  # pylint:disable=protected-access
    if group_call_id:
        if request is not None:
            raise ValueError(
                "Received multiple values for call locator. "
                "Please provide either 'group_call_id' or 'server_call_id'."
            )
        request = CallLocator(group_call_id=group_call_id, kind="groupCallLocator")
    if server_call_id:
        if request is not None:
            raise ValueError(
                "Received multiple values for call locator. "
                "Please provide either 'group_call_id' or 'server_call_id'."
            )
        request = CallLocator(server_call_id=server_call_id, kind="serverCallLocator")
    if room_id:
        if request is not None:
            raise ValueError(
                "Received multiple values for call locator. "
                "Please provide either 'group_call_id' or 'server_call_id' or 'room_id'."
            )
        request = CallLocator(room_id=room_id, kind="roomCallLocator")
    return request


def process_repeatability_first_sent(keywords: Dict[str, Any]) -> None:
    if "headers" in keywords:
        if "Repeatability-First-Sent" not in keywords["headers"]:
            keywords["headers"]["Repeatability-First-Sent"] = get_repeatability_timestamp()
    else:
        keywords["headers"] = {"Repeatability-First-Sent": get_repeatability_timestamp()}


def get_repeatability_timestamp() -> str:
    return datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S GMT")


def serialize_identifier(identifier: CommunicationIdentifier) -> CommunicationIdentifierModel:
    """Serialize the Communication identifier into CommunicationIdentifierModel

    :param identifier: Identifier object
    :type identifier: CommunicationIdentifier
    :return: CommunicationIdentifierModel
    :rtype: CommunicationIdentifierModel
    """
    try:
        request_model = CommunicationIdentifierModel(raw_id=identifier.raw_id)
        if identifier.kind == CommunicationIdentifierKind.PHONE_NUMBER:
            request_model.phone_number = PhoneNumberIdentifierModel(**identifier.properties)
        elif identifier.kind == CommunicationIdentifierKind.COMMUNICATION_USER:
            request_model.communication_user = CommunicationUserIdentifierModel(**identifier.properties)
        elif identifier.kind == CommunicationIdentifierKind.MICROSOFT_TEAMS_USER:
            request_model.microsoft_teams_user = MicrosoftTeamsUserIdentifierModel(**identifier.properties)
        elif identifier.kind == CommunicationIdentifierKind.MICROSOFT_TEAMS_APP:
            request_model.microsoft_teams_app = MicrosoftTeamsAppIdentifierModel(**identifier.properties)
        elif identifier.kind == CommunicationIdentifierKind.TEAMS_EXTENSION_USER:
            request_model.teams_extension_user = TeamsExtensionUserIdentifierModel(**identifier.properties)
        return request_model
    except AttributeError:
        raise TypeError(f"Unsupported identifier type: {identifier.__class__.__name__}") from None


def serialize_phone_identifier(identifier: Optional[PhoneNumberIdentifier]) -> Optional[PhoneNumberIdentifierModel]:
    """Serialize the Communication identifier into CommunicationIdentifierModel

    :param identifier: PhoneNumberIdentifier
    :type identifier: PhoneNumberIdentifier
    :return: PhoneNumberIdentifierModel
    :rtype: ~azure.communication.callautomation._generated.models.PhoneNumberIdentifierModel
    """
    if identifier is None:
        return None
    try:
        if identifier.kind and identifier.kind == CommunicationIdentifierKind.PHONE_NUMBER:
            request_model = PhoneNumberIdentifierModel(value=identifier.properties["value"])
            return request_model
    except AttributeError:
        pass
    raise TypeError(f"Unsupported phone identifier type: {identifier.__class__.__name__}")


def serialize_communication_user_identifier(
    identifier: Optional[CommunicationUserIdentifier],
) -> Optional[CommunicationUserIdentifierModel]:
    """Serialize the CommunicationUserIdentifier into CommunicationUserIdentifierModel

    :param identifier: CommunicationUserIdentifier
    :type identifier: CommunicationUserIdentifier
    :return: CommunicationUserIdentifierModel
    :rtype: ~azure.communication.callautomation._generated.models.CommunicationUserIdentifierModel
    """
    if identifier is None:
        return None
    try:
        if identifier.kind and identifier.kind == CommunicationIdentifierKind.COMMUNICATION_USER:
            request_model = CommunicationUserIdentifierModel(id=identifier.properties["id"])
            return request_model
    except AttributeError:
        pass
    raise TypeError(f"Unsupported user identifier type: {identifier.__class__.__name__}")


def serialize_msft_teams_app_identifier(
    identifier: Optional[MicrosoftTeamsAppIdentifier],
) -> Optional[MicrosoftTeamsAppIdentifierModel]:
    """Serialize the MicrosoftTeamsAppIdentifier into MicrosoftTeamsAppIdentifierModel

    :param identifier: MicrosoftTeamsAppIdentifier
    :type identifier: MicrosoftTeamsAppIdentifier
    :return: MicrosoftTeamsAppIdentifierModel
    :rtype: ~azure.communication.callautomation._generated.models.MicrosoftTeamsAppIdentifierModel
    """
    if identifier is None:
        return None
    try:
        if identifier.kind and identifier.kind == CommunicationIdentifierKind.MICROSOFT_TEAMS_APP:
            request_model = MicrosoftTeamsAppIdentifierModel(app_id=identifier.properties["app_id"])
            return request_model
    except AttributeError:
        pass
    raise TypeError(f"Unsupported user identifier type: {identifier.__class__.__name__}")


def deserialize_identifier(identifier_model: CommunicationIdentifierModel) -> CommunicationIdentifier:
    """
    Deserialize the CommunicationIdentifierModel into Communication Identifier

    :param identifier_model: CommunicationIdentifierModel
    :type identifier_model: CommunicationIdentifierModel
    :return: CommunicationIdentifier
    :rtype: ~azure.communication.callautomation.CommunicationIdentifier
    """
    # "raw_id" is required in responses, so we can safely assume this will not be None.
    raw_id: str = cast(str, identifier_model.raw_id)

    if identifier_model.communication_user:
        return CommunicationUserIdentifier(raw_id, raw_id=raw_id)
    if identifier_model.phone_number:
        return PhoneNumberIdentifier(identifier_model.phone_number.value, raw_id=raw_id)
    if identifier_model.microsoft_teams_user:
        return MicrosoftTeamsUserIdentifier(
            raw_id=raw_id,
            user_id=identifier_model.microsoft_teams_user.user_id,
            is_anonymous=identifier_model.microsoft_teams_user.is_anonymous,
            cloud=identifier_model.microsoft_teams_user.cloud,
        )
    if identifier_model.microsoft_teams_app:
        return MicrosoftTeamsAppIdentifier(
            raw_id=raw_id,
            app_id=identifier_model.microsoft_teams_app.app_id,
            cloud=identifier_model.microsoft_teams_app.cloud,
        )
    if identifier_model.teams_extension_user:
        return TeamsExtensionUserIdentifier(
            raw_id=raw_id,
            user_id=identifier_model.teams_extension_user.user_id,
            tenant_id=identifier_model.teams_extension_user.tenant_id,
            resource_id=identifier_model.teams_extension_user.resource_id,
            cloud=identifier_model.teams_extension_user.cloud,
        )
    return UnknownIdentifier(raw_id)


def deserialize_phone_identifier(identifier_model: PhoneNumberIdentifierModel) -> Union[PhoneNumberIdentifier, None]:
    """
    Deserialize the PhoneNumberIdentifierModel into PhoneNumberIdentifier

    :param identifier_model: PhoneNumberIdentifierModel
    :type identifier_model: PhoneNumberIdentifierModel
    :return: PhoneNumberIdentifier
    :rtype: ~azure.communication.callautomation.PhoneNumberIdentifier
    """
    if identifier_model:
        return PhoneNumberIdentifier(identifier_model.value)
    return None


def deserialize_comm_user_identifier(
    identifier_model: CommunicationUserIdentifierModel,
) -> Union[CommunicationUserIdentifier, None]:
    """
    Deserialize the CommunicationUserIdentifierModel into CommunicationUserIdentifier

    :param identifier_model: CommunicationUserIdentifierModel
    :type identifier_model: CommunicationUserIdentifierModel
    :return: CommunicationUserIdentifier
    :rtype: ~azure.communication.callautomation.CommunicationUserIdentifier
    """
    return CommunicationUserIdentifier(id=identifier_model.id) if identifier_model else None


def deserialize_msft_teams_app_identifier(
    identifier_model: MicrosoftTeamsAppIdentifierModel,
) -> Union[MicrosoftTeamsAppIdentifier, None]:
    """
    Deserialize the MicrosoftTeamsAppIdentifierModel into MicrosoftTeamsAppIdentifier

    :param identifier_model: MicrosoftTeamsAppIdentifierModel
    :type identifier_model: MicrosoftTeamsAppIdentifierModel
    :return: MicrosoftTeamsAppIdentifier
    :rtype: ~azure.communication.callautomation.MicrosoftTeamsAppIdentifier
    """
    if identifier_model:
        return MicrosoftTeamsAppIdentifier(identifier_model.app_id)
    return None



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.baremetalinfrastructure import BareMetalInfrastructureClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-baremetalinfrastructure
# USAGE
    python azure_bare_metal_storage_instances_create.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BareMetalInfrastructureClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.azure_bare_metal_storage_instances.create(
        resource_group_name="myResourceGroup",
        azure_bare_metal_storage_instance_name="myAzureBareMetalStorageInstance",
        request_body_parameters={
            "location": "westus2",
            "properties": {
                "azureBareMetalStorageInstanceUniqueIdentifier": "23415635-4d7e-41dc-9598-8194f22c24e9",
                "storageProperties": {
                    "generation": "Gen4",
                    "hardwareType": "NetApp",
                    "offeringType": "EPIC",
                    "provisioningState": "Succeeded",
                    "storageBillingProperties": {"azureBareMetalStorageInstanceSize": "", "billingMode": "PAYG"},
                    "storageType": "FC",
                    "workloadType": "ODB",
                },
            },
            "tags": {"key": "value"},
        },
    )
    print(response)


# x-ms-original-file: specification/baremetalinfrastructure/resource-manager/Microsoft.BareMetalInfrastructure/preview/2023-08-04-preview/examples/AzureBareMetalStorageInstances_Create.json
if __name__ == "__main__":
    main()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is used to execute mypy within a tox environment.

from subprocess import check_call, CalledProcessError
import argparse
import os
import logging
import sys

from ci_tools.environment_exclusions import (
    is_check_enabled, is_typing_ignored
)
from ci_tools.parsing import ParsedSetup
from ci_tools.variables import in_ci

logging.getLogger().setLevel(logging.INFO)

root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "..", ".."))

PYTHON_VERSION = "3.10"

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run mypy against target folder. ")

    parser.add_argument(
        "-t",
        "--target",
        dest="target_package",
        help="The target package directory on disk. The target module passed to run mypy will be <target_package>/azure.",
        required=True,
    )

    parser.add_argument(
        "--next",
        default=False,
        help="Next version of mypy is being tested.",
        required=False
    )

    args = parser.parse_args()
    package_dir = os.path.abspath(args.target_package)
    package_name = os.path.basename(package_dir)
    if not args.next and in_ci():
        if not is_check_enabled(args.target_package, "mypy", True) or is_typing_ignored(package_name):
            logging.info(
                f"Package {package_name} opts-out of mypy check. See https://aka.ms/python/typing-guide for information."
            )
            exit(0)

    pkg_details = ParsedSetup.from_path(package_dir)
    top_level_module = pkg_details.namespace.split(".")[0]

    commands = [
        sys.executable,
        "-m",
        "mypy",
        "--python-version",
        PYTHON_VERSION,
        "--show-error-codes",
        "--ignore-missing-imports",
    ]
    src_code = [*commands, os.path.join(args.target_package, top_level_module)]
    src_code_error = None
    sample_code_error = None
    try:
        logging.info(
            f"Running mypy commands on src code: {src_code}"
        )
        check_call(src_code)
    except CalledProcessError as src_err:
        src_code_error = src_err

    if not args.next and in_ci() and not is_check_enabled(args.target_package, "type_check_samples", True):
        logging.info(
            f"Package {package_name} opts-out of mypy check on samples."
        )
    else:
        # check if samples dir exists, if not, skip sample code check
        samples = os.path.exists(os.path.join(args.target_package, "samples"))
        generated_samples = os.path.exists(os.path.join(args.target_package, "generated_samples"))
        if not samples and not generated_samples:
            logging.info(
                f"Package {package_name} does not have a samples directory."
            )
        else:
            sample_code = [
                *commands,
                "--check-untyped-defs",
                "--follow-imports=silent",
                os.path.join(args.target_package, "samples" if samples else "generated_samples"),
            ]
            try:
                logging.info(
                    f"Running mypy commands on sample code: {sample_code}"
                )
                check_call(sample_code)
            except CalledProcessError as sample_err:
                sample_code_error = sample_err

    if args.next and in_ci() and not is_typing_ignored(package_name):
        from gh_tools.vnext_issue_creator import create_vnext_issue, close_vnext_issue
        if src_code_error or sample_code_error:
            create_vnext_issue(package_dir, "mypy")
        else:
            close_vnext_issue(package_name, "mypy")

    if src_code_error and sample_code_error:
        raise Exception(
            [
                src_code_error,
                sample_code_error,
            ],
        )
    if src_code_error:
        raise src_code_error
    if sample_code_error:
        raise sample_code_error



# coding=utf-8

from copy import deepcopy
from typing import Any, TYPE_CHECKING
from typing_extensions import Self

from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient, policies

from ._configuration import OAuth2ClientConfiguration
from ._operations import _OAuth2ClientOperationsMixin
from ._utils.serialization import Deserializer, Serializer

if TYPE_CHECKING:
    from corehttp.credentials import TokenCredential


class OAuth2Client(_OAuth2ClientOperationsMixin):  # pylint: disable=client-accepts-api-version-keyword
    """Illustrates clients generated with OAuth2 authentication.

    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~corehttp.credentials.TokenCredential
    :keyword endpoint: Service host. Default value is "http://localhost:3000".
    :paramtype endpoint: str
    """

    def __init__(
        self, credential: "TokenCredential", *, endpoint: str = "http://localhost:3000", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = OAuth2ClientConfiguration(credential=credential, endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: PipelineClient = PipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cognitiveservices.aio import CognitiveServicesManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCognitiveServicesManagementRaiBlocklistItemsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CognitiveServicesManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_get(self, resource_group):
        response = await self.client.rai_blocklist_items.get(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_item_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_create_or_update(self, resource_group):
        response = await self.client.rai_blocklist_items.create_or_update(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_item_name="str",
            rai_blocklist_item={
                "etag": "str",
                "id": "str",
                "name": "str",
                "properties": {"isRegex": bool, "pattern": "str"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_begin_delete(self, resource_group):
        response = await (
            await self.client.rai_blocklist_items.begin_delete(
                resource_group_name=resource_group.name,
                account_name="str",
                rai_blocklist_name="str",
                rai_blocklist_item_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_list(self, resource_group):
        response = self.client.rai_blocklist_items.list(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_batch_add(self, resource_group):
        response = await self.client.rai_blocklist_items.batch_add(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_items=[{"name": "str", "properties": {"isRegex": bool, "pattern": "str"}}],
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_batch_delete(self, resource_group):
        response = await self.client.rai_blocklist_items.batch_delete(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_items_names=["str"],
        )

        # please add some check logic here by yourself
        # ...



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from typing import List, Optional, Union, TYPE_CHECKING
from typing_extensions import Literal
from ._generated.models import (
    CallLocator,
    WebSocketMediaStreamingOptions as WebSocketMediaStreamingOptionsRest,
    WebSocketTranscriptionOptions as WebSocketTranscriptionOptionsRest,
    FileSource as FileSourceInternal,
    TextSource as TextSourceInternal,
    SsmlSource as SsmlSourceInternal,
    PlaySource as PlaySourceInternal,
    Choice as ChoiceInternal,
    ChannelAffinity as ChannelAffinityInternal,
    MediaStreamingSubscription as MediaStreamingSubscriptionInternal,
    TranscriptionSubscription as TranscriptionSubscriptionInternal,
)
from ._shared.models import (
    CommunicationIdentifier,
    PhoneNumberIdentifier,
)
from ._generated.models._enums import PlaySourceType
from ._generated.models._enums import RecordingStorageKind
from ._utils import (
    deserialize_phone_identifier,
    deserialize_identifier,
    deserialize_comm_user_identifier,
    serialize_identifier,
)

if TYPE_CHECKING:
    from ._generated.models._enums import (
        MediaStreamingSubscriptionState,
        StreamingTransportType,
        MediaStreamingContentType,
        MediaStreamingAudioChannelType,
        TranscriptionSubscriptionState,
        TranscriptionResultType,
        CallConnectionState,
        RecordingState,
        RecordingKind,
        VoiceKind,
        DtmfTone,
        AudioFormat,
    )
    from ._generated.models import (
        CallParticipant as CallParticipantRest,
        CallConnectionProperties as CallConnectionPropertiesRest,
        AddParticipantResponse as AddParticipantResultRest,
        RemoveParticipantResponse as RemoveParticipantResultRest,
        TransferCallResponse as TransferParticipantResultRest,
        RecordingStateResponse as RecordingStateResultRest,
        MuteParticipantsResult as MuteParticipantsResultRest,
        SendDtmfTonesResult as SendDtmfTonesResultRest,
        CancelAddParticipantResponse as CancelAddParticipantResultRest,
    )


class CallInvite:
    """Details of call invitation for outgoing call.

    **DEPRECATED**: This model has been deprecated and will be removed from future releases.
    Please pass in the target `CommunicateIdentifier` directly.

    :param target: Target's identity.
    :type target: ~azure.communication.callautomation.CommunicationIdentifier
    :keyword source_caller_id_number: Caller's phone number identifier.
     Required for PSTN outbound call.
    :paramtype source_caller_id_number: ~azure.communication.callautomation.PhoneNumberIdentifier
    :keyword source_display_name: Set display name for caller
    :paramtype source_display_name: str
    """

    target: CommunicationIdentifier
    """Target's identity."""
    source_caller_id_number: Optional[PhoneNumberIdentifier]
    """ Caller's phone number identifier. Required for PSTN outbound call."""
    source_display_name: Optional[str]
    """Display name for caller"""

    def __init__(  # pylint: disable=unused-argument
        self,
        target: CommunicationIdentifier,
        *,
        source_caller_id_number: Optional[PhoneNumberIdentifier] = None,
        source_display_name: Optional[str] = None,
        **kwargs,
    ):
        self.target = target
        self.source_caller_id_number = source_caller_id_number
        self.source_display_name = source_display_name


class RoomCallLocator:
    """The locator to locate ongoing call, using room id.
    **DEPRECATED**: This model has been deprecated and will be removed from future releases.
    Please pass in the `room_id` directly.
    :param room_id: The room id of ongoing call.
    :type room_id: str
    """

    room_id: str
    """The acs room id of ongoing call."""
    kind: str = "roomCallLocator"
    """This is for locating the call with acs room id."""

    def __init__(self, room_id: str, **kwargs):  # pylint: disable=unused-argument
        self.room_id = room_id
        self.kind = "roomCallLocator"

    def _to_generated(self):
        return CallLocator(kind=self.kind, room_id=self.room_id)


class ServerCallLocator:
    """The locator to locate ongoing call, using server call id.

    **DEPRECATED**: This model has been deprecated and will be removed from future releases.
    Please pass in the `server_call_id` directly.

    :param server_call_id: The server call id of ongoing call.
    :type server_call_id: str
    """

    server_call_id: str
    """The server call id of ongoing call."""
    kind: str = "serverCallLocator"
    """This is for locating the call with server call id."""

    def __init__(self, server_call_id: str, **kwargs):  # pylint: disable=unused-argument
        self.server_call_id = server_call_id
        self.kind = "serverCallLocator"

    def _to_generated(self):
        return CallLocator(kind=self.kind, server_call_id=self.server_call_id)


class GroupCallLocator:
    """The locator to locate ongoing call, using group call id.

    **DEPRECATED**: This model has been deprecated and will be removed from future releases.
    Please pass in the `group_call_id` directly.

    :param group_call_id: The group call id of ongoing call.
    :type group_call_id: str
    """

    group_call_id: str
    """The group call id of ongoing call."""
    kind: str = "groupCallLocator"
    """This is for locating the call with group call id."""

    def __init__(self, group_call_id: str, **kwargs):  # pylint: disable=unused-argument
        self.group_call_id = group_call_id
        self.kind = "groupCallLocator"

    def _to_generated(self):
        return CallLocator(kind=self.kind, group_call_id=self.group_call_id)


class RecordingStorage:
    """Recording Storage for the recording.
    :param kind: Defines the kind of external storage.
    :type kind: str
    """

    kind: str
    """The recording storage kind"""


class AzureCommunicationsRecordingStorage(RecordingStorage):
    """
    Recording Storage for the recording.
    :param kind: Defines the kind of external storage.
    :type kind: ~azure.communication.callautomation.RecordingStorageKind or str
    """

    # This is intended to be read-only
    kind: Literal[  # type: ignore[reportIncompatibleVariableOverride]
        RecordingStorageKind.AZURE_COMMUNICATION_SERVICES
    ] = RecordingStorageKind.AZURE_COMMUNICATION_SERVICES
    """The kind of recording storage is set to AZURE_COMMUNICATION_SERVICES"""


class AzureBlobContainerRecordingStorage(RecordingStorage):
    """Recording Storage for the recording.
    :param kind: Defines the kind of external storage.
    :type kind: ~azure.communication.callautomation.RecordingStorageKind or str
    :param container_url: Defines the kind of external storage. Required.
    :type container_url: str
    """

    # This is intended to be read-only
    kind: Literal[RecordingStorageKind.AZURE_BLOB_STORAGE] = (  # type: ignore[reportIncompatibleVariableOverride]
        RecordingStorageKind.AZURE_BLOB_STORAGE
    )
    """The kind of recording storage is set to AZURE_BLOB_STORAGE"""
    container_url: str
    """The container url for the AZURE_BLOB_STORAGE type"""

    def __init__(self, container_url: str):
        self.container_url = container_url


class ChannelAffinity:
    """Channel affinity for a participant.

    All required parameters must be populated in order to send to Azure.

    :param target_participant: The identifier for the participant whose bitstream will be written to the
     channel represented by the channel number. Required.
    :type target_participant: ~azure.communication.callautomation.CommunicationIdentifier
    :param channel: Channel number to which bitstream from a particular participant will be written.
    :type channel: int
    """

    target_participant: CommunicationIdentifier
    """The identifier for the participant whose bitstream will be written to the
     channel represented by the channel number. Required."""
    channel: int
    """ Channel number to which bitstream from a particular participant will be written."""

    def __init__(self, target_participant: CommunicationIdentifier, channel: int):
        self.target_participant = target_participant
        self.channel = channel

    def _to_generated(self):
        return ChannelAffinityInternal(participant=serialize_identifier(self.target_participant), channel=self.channel)


class FileSource:
    """Media file source of URL to be played in action such as Play media.

    :param url: Url for the audio file to be played.
    :type url: str
    :keyword play_source_cache_id: Cached source id of the play media, if it exists.
    :paramtype play_source_cache_id: str
    """

    url: str
    """Url for the audio file to be played."""
    play_source_cache_id: Optional[str]
    """Cached source id of the play media, if it exists."""

    def __init__(self, url: str, *, play_source_cache_id: Optional[str] = None):
        self.url = url
        self.play_source_cache_id = play_source_cache_id

    def _to_generated(self):
        return PlaySourceInternal(
            kind=PlaySourceType.FILE,
            file=FileSourceInternal(uri=self.url),
            play_source_cache_id=self.play_source_cache_id,
        )


class TextSource:
    """TextSource to be played in actions such as Play media.

    :keyword text: Text for the cognitive service to be played. Required.
    :paramtype text: str
    :keyword source_locale: Source language locale to be played. Refer to available locales here:
        https://learn.microsoft.com/azure/cognitive-services/speech-service/language-support?tabs=stt-tts
    :paramtype source_locale: str
    :keyword voice_kind: Voice kind type. Known values are: "male" and "female".
    :paramtype voice_kind: str or ~azure.communication.callautomation.VoiceKind
    :keyword voice_name: Voice name to be played. Refer to available Text-to-speech voices here:
        https://learn.microsoft.com/azure/cognitive-services/speech-service/language-support?tabs=stt-tts
    :paramtype voice_name: str
    :keyword play_source_cache_id: Cached source id of the play media, if it exists.
    :paramtype play_source_cache_id: str
    :keyword custom_voice_endpoint_id: Endpoint where the custom voice was deployed.
    :paramtype custom_voice_endpoint_id: str
    """

    text: str
    """Text for the cognitive service to be played."""
    source_locale: Optional[str]
    """Source language locale to be played. Refer to available locales here:
        https://learn.microsoft.com/azure/cognitive-services/speech-service/language-support?tabs=stt-tts"""
    voice_kind: Optional[Union[str, "VoiceKind"]]
    """Voice kind type. Known values are: "male" and "female"."""
    voice_name: Optional[str]
    """Voice name to be played. Refer to available Text-to-speech voices here:
        https://learn.microsoft.com/azure/cognitive-services/speech-service/language-support?tabs=stt-tts"""
    play_source_cache_id: Optional[str]
    """Cached source id of the play media, if it exists."""
    custom_voice_endpoint_id: Optional[str]
    """Endpoint where the custom voice was deployed."""

    def __init__(
        self,
        *,
        text: str,
        source_locale: Optional[str] = None,
        voice_kind: Optional[Union[str, "VoiceKind"]] = None,
        voice_name: Optional[str] = None,
        play_source_cache_id: Optional[str] = None,
        custom_voice_endpoint_id: Optional[str] = None,
    ):
        self.text = text
        self.source_locale = source_locale
        self.voice_kind = voice_kind
        self.voice_name = voice_name
        self.play_source_cache_id = play_source_cache_id
        self.custom_voice_endpoint_id = custom_voice_endpoint_id

    def _to_generated(self):
        return PlaySourceInternal(
            kind=PlaySourceType.TEXT,
            text=TextSourceInternal(
                text=self.text,
                source_locale=self.source_locale,
                voice_kind=self.voice_kind,
                voice_name=self.voice_name,
                custom_voice_endpoint_id=self.custom_voice_endpoint_id,
            ),
            play_source_cache_id=self.play_source_cache_id,
        )


class SsmlSource:
    """SsmlSource to be played in actions such as Play media.

    :keyword ssml_text: Ssml string for the cognitive service to be played. Required.
    :paramtype ssml_text: str
    :keyword play_source_cache_id: Cached source id of the play media, if it exists.
    :paramtype play_source_cache_id: str
    :keyword custom_voice_endpoint_id: Endpoint id where the custom voice model is deployed.
    :paramtype custom_voice_endpoint_id: str
    """

    ssml_text: str
    """Ssml string for the cognitive service to be played."""
    play_source_cache_id: Optional[str]
    """Cached source id of the play media, if it exists."""
    custom_voice_endpoint_id: Optional[str]
    """Endpoint where the custom voice model was deployed."""

    def __init__(
        self,
        *,
        ssml_text: str,
        play_source_cache_id: Optional[str] = None,
        custom_voice_endpoint_id: Optional[str] = None,
    ):
        self.ssml_text = ssml_text
        self.play_source_cache_id = play_source_cache_id
        self.custom_voice_endpoint_id = custom_voice_endpoint_id

    def _to_generated(self):
        return PlaySourceInternal(
            kind=PlaySourceType.SSML,
            ssml=SsmlSourceInternal(ssml_text=self.ssml_text, custom_voice_endpoint_id=self.custom_voice_endpoint_id),
            play_source_cache_id=self.play_source_cache_id,
        )


class MediaStreamingOptions:
    """Configuration of Media streaming.

    :param transport_url: Transport URL for media streaming.
    :type transport_url: str
    :param transport_type: The type of transport to be used for media streaming.
    :type transport_type: str or ~azure.communication.callautomation.StreamingTransport
    :param content_type: Content type to stream, eg. audio, audio/video.
    :type content_type: str or ~azure.communication.callautomation.MediaStreamingContentType
    :param audio_channel_type: Audio channel type to stream, eg. unmixed audio, mixed audio.
    :type audio_channel_type: str or ~azure.communication.callautomation.MediaStreamingAudioChannelType
    :keyword start_media_streaming: Determines if the media streaming should be started immediately
     after call is answered or not. Required.
    :paramtype start_media_streaming: bool
    :param enable_bidirectional: A value indicating whether bidirectional streaming is enabled
    :type enable_bidirectional: bool
    :param audio_format: Specifies the audio format used for encoding
    :type audio_format: ~azure.communication.callautomation.AudioFormat
    """

    transport_url: str
    """Transport URL for media streaming."""
    transport_type: Union[str, "StreamingTransportType"]
    """The type of transport to be used for media streaming."""
    content_type: Union[str, "MediaStreamingContentType"]
    """Content type to stream, eg. audio, audio/video."""
    audio_channel_type: Union[str, "MediaStreamingAudioChannelType"]
    """Audio channel type to stream, eg. unmixed audio, mixed audio."""
    start_media_streaming: Optional[bool] = None
    """Determines if the media streaming should be started immediately
     after call is answered or not"""
    enable_bidirectional: Optional[bool] = None
    """A value indicating whether bidirectional streaming is enabled"""
    audio_format: Optional[Union[str, "AudioFormat"]] = None
    """Specifies the audio format used for encoding."""
    enable_dtmf_tones: Optional[bool] = None
    """A value that indicates whether to stream the DTMF tones."""

    def __init__(
        self,
        *,
        transport_url: str,
        transport_type: Union[str, "StreamingTransportType"],
        content_type: Union[str, "MediaStreamingContentType"],
        audio_channel_type: Union[str, "MediaStreamingAudioChannelType"],
        start_media_streaming: Optional[bool] = None,
        enable_bidirectional: Optional[bool] = None,
        audio_format: Optional[Union[str, "AudioFormat"]] = None,
        enable_dtmf_tones: Optional[bool] = None,
    ):
        self.transport_url = transport_url
        self.transport_type = transport_type
        self.content_type = content_type
        self.audio_channel_type = audio_channel_type
        self.start_media_streaming = start_media_streaming
        self.enable_bidirectional = enable_bidirectional
        self.audio_format = audio_format
        self.enable_dtmf_tones = enable_dtmf_tones

    def _to_generated(self):
        return WebSocketMediaStreamingOptionsRest(
            transport_url=self.transport_url,
            transport_type=self.transport_type,
            content_type=self.content_type,
            audio_channel_type=self.audio_channel_type,
            start_media_streaming=self.start_media_streaming,
            enable_bidirectional=self.enable_bidirectional,
            audio_format=self.audio_format,
            enable_dtmf_tones=self.enable_dtmf_tones,
        )


class TranscriptionOptions:
    """Configuration of live transcription.

    :param transport_url: Transport URL for live transcription. Required.
    :type transport_url: str
    :param transport_type: The type of transport to be used for live transcription, eg. Websocket.
     Required. "websocket"
    :type transport_type: str or ~azure.communication.callautomation.StreamingTransportType
    :param locale: Defines the locale for the data e.g en-CA, en-AU. Required.
    :type locale: str
    :param start_transcription: Determines if the transcription should be started immediately after
     call is answered or not. Required.
    :type start_transcription: bool
    :param speech_recognition_model_endpoint_id: Endpoint where the custom model was deployed
    :type speech_recognition_model_endpoint_id: str
    :param enable_intermediate_results: Enables intermediate results for the transcribed speech
    :type enable_intermediate_results: bool
    """

    transport_url: str
    """Transport URL for live transcription."""
    transport_type: Union[str, "StreamingTransportType"]
    """The type of transport to be used for live transcription."""
    locale: str
    """Defines the locale for the data."""
    start_transcription: bool
    """Determines if the transcription should be started immediately after call is answered or not."""
    speech_recognition_model_endpoint_id: Optional[str] = None
    """Endpoint where the custom model was deployed."""
    enable_intermediate_results: Optional[bool] = None
    """Enables intermediate results for the transcribed speech."""

    def __init__(
        self,
        *,
        transport_url: str,
        transport_type: Union[str, "StreamingTransportType"],
        locale: str,
        start_transcription: bool,
        speech_recognition_model_endpoint_id: Optional[str] = None,
        enable_intermediate_results: Optional[bool] = None,
    ):
        self.transport_url = transport_url
        self.transport_type = transport_type
        self.locale = locale
        self.start_transcription = start_transcription
        self.speech_recognition_model_endpoint_id = speech_recognition_model_endpoint_id
        self.enable_intermediate_results = enable_intermediate_results

    def _to_generated(self):
        return WebSocketTranscriptionOptionsRest(
            transport_url=self.transport_url,
            transport_type=self.transport_type,
            locale=self.locale,
            start_transcription=self.start_transcription,
            speech_recognition_model_endpoint_id=self.speech_recognition_model_endpoint_id,
            enable_intermediate_results=self.enable_intermediate_results,
        )


class MediaStreamingSubscription:
    """Media streaming Subscription Object.

    :keyword id: Subscription Id.
    :paramtype id: str
    :keyword state: Media streaming subscription state. Known values are: "disabled", "inactive", and
     "active".
    :paramtype state: str or
     ~azure.communication.callautomation.MediaStreamingSubscriptionState
    :keyword subscribed_content_types: Subscribed media streaming content types.
    :paramtype subscribed_content_types: list[str or
     ~azure.communication.callautomation.MediaStreamingContentType]
    """

    id: Optional[str]
    """Subscription Id."""
    state: Optional[Union[str, "MediaStreamingSubscriptionState"]]
    """Media streaming subscription state."""
    subscribed_content_types: Optional[List[Union[str, "MediaStreamingContentType"]]]
    """Subscribed media streaming content types."""

    def __init__(
        self,
        *,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
        state: Optional[Union[str, "MediaStreamingSubscriptionState"]] = None,
        subscribed_content_types: Optional[List[Union[str, "MediaStreamingContentType"]]] = None,
    ) -> None:
        """
        :keyword id: Subscription Id.
        :paramtype id: str
        :keyword state: Media streaming subscription state. Known values are: "disabled", "inactive",
         and "active".
        :paramtype state: str or
         ~azure.communication.callautomation.MediaStreamingSubscriptionState
        :keyword subscribed_content_types: Subscribed media streaming content types.
        :paramtype subscribed_content_types: list[str or
         ~azure.communication.callautomation.MediaStreamingContentType]
        """

        self.id = id
        self.state = state
        self.subscribed_content_types = subscribed_content_types

    def _to_generated(self):
        return MediaStreamingSubscriptionInternal(
            id=self.id,
            state=self.state,
            subscribed_content_types=self.subscribed_content_types,
        )


class TranscriptionSubscription:
    """Transcription Subscription Object.

    :keyword id: Subscription Id.
    :paramtype id: str
    :keyword state: Transcription subscription state. Known values are: "disabled", "inactive", and
     "active".
    :paramtype state: str or
     ~azure.communication.callautomation.TranscriptionSubscriptionState
    :keyword subscribed_result_types: Subscribed transcription result types.
    :paramtype subscribed_result_types: list[str or
     ~azure.communication.callautomation.TranscriptionResultType]
    """

    id: Optional[str]
    """Subscription Id."""
    state: Optional[Union[str, "TranscriptionSubscriptionState"]]
    """Transcription subscription state."""
    subscribed_result_types: Optional[List[Union[str, "TranscriptionResultType"]]]
    """Subscribed transcription result types."""
    locale: Optional[str]
    """Specifies the locale used for transcription, e.g., en-CA or en-AU."""

    def __init__(
        self,
        *,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
        state: Optional[Union[str, "TranscriptionSubscriptionState"]] = None,
        subscribed_result_types: Optional[List[Union[str, "TranscriptionResultType"]]] = None,
        locale: Optional[str] = None,
    ) -> None:
        self.id = id
        self.state = state
        self.subscribed_result_types = subscribed_result_types
        self.locale = locale

    def _to_generated(self):
        return TranscriptionSubscriptionInternal(
            id=self.id, state=self.state, subscribed_result_types=self.subscribed_result_types, locale=self.locale
        )


class CallConnectionProperties:  # pylint: disable=too-many-instance-attributes
    """Detailed properties of the call.

    :keyword call_connection_id: The call connection id of this call leg.
    :paramtype call_connection_id: str
    :keyword server_call_id: The server call id of this call.
    :paramtype server_call_id: str
    :keyword targets: The targets of the call when the call was originally initiated.
    :paramtype targets: list[~azure.communication.callautomation.CommunicationIdentifier]
    :keyword call_connection_state: The state of the call.
    :paramtype call_connection_state: str or ~azure.communication.callautomation.CallConnectionState
    :keyword callback_url: The callback URL.
    :paramtype callback_url: str
    :keyword source_caller_id_number:
     The source caller Id, a phone number, that's shown to the
     PSTN participant being invited.
     Required only when calling a PSTN callee.
    :paramtype source_caller_id_number: ~azure.communication.callautomation.PhoneNumberIdentifier
    :keyword source_display_name:  Display name of the call if dialing out.
    :paramtype source_display_name: str
    :keyword source: Source identity of the caller.
    :paramtype source: ~azure.communication.callautomation.CommunicationIdentifier
    :keyword correlation_id: Correlation ID of the call
    :paramtype correlation_id: str
    :keyword answered_by: The identifier that answered the call
    :paramtype answered_by: ~azure.communication.callautomation.CommunicationUserIdentifier
    :keyword media_streaming_subscription: media_streaming_subscription for media streaming.
    :paramtype media_streaming_subscription: ~azure.communication.callautomation.MediaStreamingSubscription
    :keyword transcription_subscription: transcription_subscription for transcription.
    :paramtype transcription_subscription: ~azure.communication.callautomation.TranscriptionSubscription
    :keyword answered_for: Identity of the original Pstn target of an incoming Call. Only populated
     when the original target is a Pstn number.
    :paramtype answered_for: ~azure.communication.callautomation.PhoneNumberIdentifier
    """

    call_connection_id: Optional[str]
    """The call connection id of this call leg."""
    server_call_id: Optional[str]
    """The server call id of this call."""
    targets: Optional[List[CommunicationIdentifier]]
    """The targets of the call when the call was originally initiated."""
    call_connection_state: Optional[Union[str, "CallConnectionState"]]
    """The state of the call."""
    callback_url: Optional[str]
    """The callback URL."""
    source_caller_id_number: Optional[PhoneNumberIdentifier]
    """The source caller Id, a phone number, that's shown to the
     PSTN participant being invited.
     Required only when calling a PSTN callee."""
    source_display_name: Optional[str]
    """Display name of the call if dialing out."""
    source: Optional[CommunicationIdentifier]
    """Source identity of the caller."""
    correlation_id: Optional[str]
    """Correlation ID of the call"""
    media_streaming_subscription: Optional[MediaStreamingSubscription]
    """Media streaming subscription."""
    transcription_subscription: Optional[TranscriptionSubscription]
    """Transcription subscription."""
    answered_by: Optional[CommunicationIdentifier]
    """The identifier that answered the call"""
    answered_for: Optional[PhoneNumberIdentifier]
    """The phone identifier that answered the call"""

    def __init__(
        self,
        *,
        call_connection_id: Optional[str] = None,
        server_call_id: Optional[str] = None,
        targets: Optional[List[CommunicationIdentifier]] = None,
        call_connection_state: Optional[Union[str, "CallConnectionState"]] = None,
        callback_url: Optional[str] = None,
        source_caller_id_number: Optional[PhoneNumberIdentifier] = None,
        source_display_name: Optional[str] = None,
        source: Optional[CommunicationIdentifier] = None,
        correlation_id: Optional[str] = None,
        answered_by: Optional[CommunicationIdentifier] = None,
        media_streaming_subscription: Optional[MediaStreamingSubscription] = None,
        transcription_subscription: Optional[TranscriptionSubscription] = None,
        answered_for: Optional[PhoneNumberIdentifier] = None,
    ):
        self.call_connection_id = call_connection_id
        self.server_call_id = server_call_id
        self.targets = targets
        self.call_connection_state = call_connection_state
        self.callback_url = callback_url
        self.source_caller_id_number = source_caller_id_number
        self.source_display_name = source_display_name
        self.source = source
        self.correlation_id = correlation_id
        self.answered_by = answered_by
        self.media_streaming_subscription = media_streaming_subscription
        self.transcription_subscription = transcription_subscription
        self.answered_for = answered_for

    @classmethod
    def _from_generated(cls, call_connection_properties_generated: "CallConnectionPropertiesRest"):
        target_models = []
        if call_connection_properties_generated.targets:
            for target in call_connection_properties_generated.targets:
                target_models.append(deserialize_identifier(target))

        media_streaming: Optional[MediaStreamingSubscription] = None
        if call_connection_properties_generated.media_streaming_subscription:
            media_streaming = MediaStreamingSubscription(
                id=call_connection_properties_generated.media_streaming_subscription.id,
                state=call_connection_properties_generated.media_streaming_subscription.state,
                subscribed_content_types=call_connection_properties_generated.media_streaming_subscription.subscribed_content_types,  # pylint:disable=line-too-long
            )
        transcription: Optional[TranscriptionSubscription] = None
        if call_connection_properties_generated.transcription_subscription:
            transcription = TranscriptionSubscription(
                id=call_connection_properties_generated.transcription_subscription.id,
                state=call_connection_properties_generated.transcription_subscription.state,
                subscribed_result_types=call_connection_properties_generated.transcription_subscription.subscribed_result_types,  # pylint:disable=line-too-long
                locale=call_connection_properties_generated.transcription_subscription.locale,
            )
        return cls(
            call_connection_id=call_connection_properties_generated.call_connection_id,
            server_call_id=call_connection_properties_generated.server_call_id,
            targets=target_models,
            call_connection_state=call_connection_properties_generated.call_connection_state,
            callback_url=call_connection_properties_generated.callback_uri,
            source_caller_id_number=(
                deserialize_phone_identifier(call_connection_properties_generated.source_caller_id_number)
                if call_connection_properties_generated.source_caller_id_number
                else None
            ),
            source_display_name=call_connection_properties_generated.source_display_name,
            source=(
                deserialize_identifier(call_connection_properties_generated.source)
                if call_connection_properties_generated.source
                else None
            ),
            correlation_id=call_connection_properties_generated.correlation_id,
            answered_by=(
                deserialize_comm_user_identifier(call_connection_properties_generated.answered_by)
                if call_connection_properties_generated.answered_by
                else None
            ),
            media_streaming_subscription=media_streaming,
            transcription_subscription=transcription,
            answered_for=(
                deserialize_phone_identifier(call_connection_properties_generated.answered_for)
                if call_connection_properties_generated.answered_for
                else None
            ),
        )


class RecordingProperties:
    """Detailed recording properties of the call.

    :keyword recording_id: Id of this recording operation.
    :paramtype recording_id: str
    :keyword recording_state: state of ongoing recording.
    :paramtype recording_state: str or ~azure.communication.callautomation.RecordingState
    :keyword recording_kind: kind of the recording.
    :paramtype recording_kind: str or ~azure.communication.callautomation.RecordingKind
    """

    recording_id: Optional[str]
    """Id of this recording operation."""
    recording_state: Optional[Union[str, "RecordingState"]]
    """state of ongoing recording."""
    recording_kind: Optional[Union[str, "RecordingKind"]]
    """kind of the recording."""

    def __init__(
        self,
        *,
        recording_id: Optional[str] = None,
        recording_state: Optional[Union[str, "RecordingState"]] = None,
        recording_kind: Optional[Union[str, "RecordingKind"]] = None,
    ):
        self.recording_id = recording_id
        self.recording_state = recording_state
        self.recording_kind = recording_kind

    @classmethod
    def _from_generated(cls, recording_state_result: "RecordingStateResultRest"):
        return cls(
            recording_id=recording_state_result.recording_id,
            recording_state=recording_state_result.recording_state,
            recording_kind=recording_state_result.recording_kind,
        )


class CallParticipant:
    """Details of an Azure Communication Service call participant.

    :keyword identifier: Communication identifier of the participant.
    :paramtype identifier: ~azure.communication.callautomation.CommunicationIdentifier
    :keyword is_muted: Is participant muted.
    :paramtype is_muted: bool
    :keyword is_on_hold: Is participant on hold.
    :paramtype is_on_hold: bool
    """

    identifier: Optional[CommunicationIdentifier]
    """Communication identifier of the participant."""
    is_muted: Optional[bool]
    """Is participant muted."""
    is_on_hold: Optional[bool]
    """Is participant on hold."""

    def __init__(
        self,
        *,
        identifier: Optional[CommunicationIdentifier] = None,
        is_muted: Optional[bool] = False,
        is_on_hold: Optional[bool] = False,
    ):
        self.identifier = identifier
        self.is_muted = is_muted
        self.is_on_hold = is_on_hold

    @classmethod
    def _from_generated(cls, call_participant_generated: "CallParticipantRest"):
        return cls(
            identifier=(
                deserialize_identifier(call_participant_generated.identifier)
                if call_participant_generated.identifier
                else None
            ),
            is_muted=call_participant_generated.is_muted,
            is_on_hold=call_participant_generated.is_on_hold,
        )


class AddParticipantResult:
    """The result payload for adding participants to the call.

    :keyword participant: Participant that was added with this request.
    :paramtype participant: ~azure.communication.callautomation.CallParticipant
    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    :keyword invitation_id: Invitation ID used to add a participant.
    :paramtype invitation_id: str
    """

    invitation_id: Optional[str]
    """Invitation ID used to add participant."""
    participant: Optional[CallParticipant]
    """Participant that was added with this request."""
    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(
        self,
        *,
        invitation_id: Optional[str] = None,
        participant: Optional[CallParticipant] = None,
        operation_context: Optional[str] = None,
    ):
        self.invitation_id = invitation_id
        self.participant = participant
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, add_participant_result_generated: "AddParticipantResultRest"):
        return cls(
            invitation_id=add_participant_result_generated.invitation_id,
            participant=(
                CallParticipant._from_generated(  # pylint:disable=protected-access
                    add_participant_result_generated.participant
                )
                if add_participant_result_generated.participant
                else None
            ),
            operation_context=add_participant_result_generated.operation_context,
        )


class RemoveParticipantResult:
    """The response payload for removing participants of the call.

    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    """

    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(self, *, operation_context: Optional[str] = None) -> None:
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, remove_participant_result_generated: "RemoveParticipantResultRest"):
        return cls(operation_context=remove_participant_result_generated.operation_context)


class TransferCallResult:
    """The response payload for transferring the call.

    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    """

    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(self, *, operation_context: Optional[str] = None) -> None:
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, transfer_result_generated: "TransferParticipantResultRest"):
        return cls(operation_context=transfer_result_generated.operation_context)


class RecognitionChoice:
    """
    An IVR choice for the recognize operation.

    :keyword label: Identifier for a given choice. Required.
    :paramtype label: str
    :keyword phrases: List of phrases to recognize. Required.
    :paramtype phrases: list[str]
    :keyword tone: Known values are: "zero", "one", "two", "three", "four", "five", "six", "seven",
     "eight", "nine", "a", "b", "c", "d", "pound", and "asterisk".
    :paramtype tone: str or ~azure.communication.callautomation.DtmfTone
    """

    label: str
    """Identifier for a given choice."""
    phrases: List[str]
    """List of phrases to recognize."""
    tone: Optional[Union[str, "DtmfTone"]]
    """Known values are: "zero", "one", "two", "three", "four", "five", "six", "seven",
     "eight", "nine", "a", "b", "c", "d", "pound", and "asterisk"."""

    def __init__(self, *, label: str, phrases: List[str], tone: Optional[Union[str, "DtmfTone"]] = None):
        self.label = label
        self.phrases = phrases
        self.tone = tone

    def _to_generated(self):
        return ChoiceInternal(label=self.label, phrases=self.phrases, tone=self.tone)


class MuteParticipantResult:
    """The result payload for muting participant from the call.

    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    """

    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(self, *, operation_context: Optional[str] = None) -> None:
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, mute_participant_result_generated: "MuteParticipantsResultRest"):
        return cls(operation_context=mute_participant_result_generated.operation_context)


class SendDtmfTonesResult:
    """The result payload for send Dtmf tones.
    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    """

    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(self, *, operation_context: Optional[str] = None) -> None:
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, send_dtmf_tones_result_generated: "SendDtmfTonesResultRest"):
        return cls(operation_context=send_dtmf_tones_result_generated.operation_context)


class CancelAddParticipantOperationResult:
    """The result payload for cancelling add participant request for a participant.
    :keyword invitation_id: Invitation ID that was used to add the participant to the call.
    :paramtype participant: str
    :keyword operation_context: The operation context provided by client.
    :paramtype operation_context: str
    """

    invitation_id: Optional[str]
    """Invitation ID that was used to add the participant to the call."""
    operation_context: Optional[str]
    """The operation context provided by client."""

    def __init__(self, *, invitation_id: Optional[str] = None, operation_context: Optional[str] = None):
        self.invitation_id = invitation_id
        self.operation_context = operation_context

    @classmethod
    def _from_generated(cls, cancel_add_participant_operation_result_generated: "CancelAddParticipantResultRest"):
        return cls(
            invitation_id=cancel_add_participant_operation_result_generated.invitation_id,
            operation_context=cancel_add_participant_operation_result_generated.operation_context,
        )



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import functools
import logging
from time import sleep

# let this import fail, this package is used intentionally without the presence of azure-core
try:
    from azure.core.exceptions import ResourceExistsError
except:
    pass

try:
    # Note: these models are only available from v17.0.0 and higher, if you need them you'll also need azure-core 1.4.0 and higher
    from azure.mgmt.storage import StorageManagementClient
    from azure.mgmt.storage.models import (
        StorageAccount,
        Endpoints,
        LastAccessTimeTrackingPolicy,
        BlobServiceProperties,
        DeleteRetentionPolicy,
    )
except ImportError:
    pass

from . import AzureMgmtPreparer, ResourceGroupPreparer, FakeResource
from .exceptions import AzureTestError
from .resource_testcase import RESOURCE_GROUP_PARAM
from .sanitizers import add_general_string_sanitizer


FakeStorageAccount = FakeResource


# Storage Account Preparer and its shorthand decorator
class StorageAccountPreparer(AzureMgmtPreparer):
    def __init__(
        self,
        name_prefix="",
        sku="Standard_LRS",
        location="westus",
        kind="StorageV2",
        parameter_name="storage_account",
        resource_group_parameter_name=RESOURCE_GROUP_PARAM,
        disable_recording=True,
        playback_fake_resource=None,
        client_kwargs=None,
        random_name_enabled=False,
        use_cache=False,
    ):
        super(StorageAccountPreparer, self).__init__(
            name_prefix,
            24,
            disable_recording=disable_recording,
            playback_fake_resource=playback_fake_resource,
            client_kwargs=client_kwargs,
            random_name_enabled=random_name_enabled,
        )
        self.location = location
        self.sku = sku
        self.kind = kind
        self.resource_group_parameter_name = resource_group_parameter_name
        self.parameter_name = parameter_name
        self.storage_key = ""
        self.resource_moniker = self.name_prefix
        self.set_cache(use_cache, sku, location, name_prefix)
        if random_name_enabled:
            self.resource_moniker += "storname"

    def create_resource(self, name, **kwargs):
        if self.is_live:
            self.client = self.create_mgmt_client(StorageManagementClient)
            group = self._get_resource_group(**kwargs)
            storage_async_operation = self._create_account(group.name, name)

            self.resource = storage_async_operation.result()
            storage_keys = {v.key_name: v.value for v in self.client.storage_accounts.list_keys(group.name, name).keys}
            self.storage_key = storage_keys["key1"]

            add_general_string_sanitizer(target=name, value=self.resource_moniker)
        else:
            self.resource = StorageAccount(
                location=self.location,
            )
            self.resource.name = name
            self.resource.id = name
            self.resource.primary_endpoints = Endpoints()
            self.resource.primary_endpoints.blob = "https://{}.{}.core.windows.net".format(name, "blob")
            self.resource.primary_endpoints.queue = "https://{}.{}.core.windows.net".format(name, "queue")
            self.resource.primary_endpoints.table = "https://{}.{}.core.windows.net".format(name, "table")
            self.resource.primary_endpoints.file = "https://{}.{}.core.windows.net".format(name, "file")
            self.storage_key = "ZmFrZV9hY29jdW50X2tleQ=="
        return {
            self.parameter_name: self.resource,
            "{}_key".format(self.parameter_name): self.storage_key,
            "{}_cs".format(self.parameter_name): ";".join(
                [
                    "DefaultEndpointsProtocol=https",
                    "AccountName={}".format(name),
                    "AccountKey={}".format(self.storage_key),
                    "BlobEndpoint={}".format(self.resource.primary_endpoints.blob),
                    "TableEndpoint={}".format(self.resource.primary_endpoints.table),
                    "QueueEndpoint={}".format(self.resource.primary_endpoints.queue),
                    "FileEndpoint={}".format(self.resource.primary_endpoints.file),
                ]
            ),
        }

    def remove_resource(self, name, **kwargs):
        if self.is_live:
            retries = 3
            for _ in range(retries):
                try:
                    group = self._get_resource_group(**kwargs)
                    self.client.storage_accounts.delete(group.name, name)
                except ResourceExistsError as e:
                    # Occassionally a storage test will try to delete the
                    # resource before the previous operation has been completed
                    logger = logging.getLogger()
                    logger.warning(
                        "An error occurred while trying to delete storage account {}. Waiting \
                            ten seconds and retrying the delete.".format(
                            self.resource.name
                        )
                    )
                    sleep(30)

    def _create_account(self, resource_group_name, account_name):
        return self.client.storage_accounts.begin_create(
            resource_group_name,
            account_name,
            {
                "sku": {"name": self.sku},
                "location": self.location,
                "kind": self.kind,
                "enable_https_traffic_only": True,
            },
        )

    def _get_resource_group(self, **kwargs):
        try:
            return kwargs.get(self.resource_group_parameter_name)
        except KeyError:
            template = (
                "To create a storage account a resource group is required. Please add "
                "decorator @{} in front of this storage account preparer."
            )
            raise AzureTestError(template.format(ResourceGroupPreparer.__name__))


class BlobAccountPreparer(StorageAccountPreparer):
    def __init__(self, **kwargs):
        self.is_versioning_enabled = kwargs.pop("is_versioning_enabled", None)
        self.is_last_access_time_enabled = kwargs.pop("is_last_access_time_enabled", None)
        self.container_retention_days = kwargs.pop("container_retention_days", None)

        super(BlobAccountPreparer, self).__init__(**kwargs)

    def _create_account(self, resource_group_name, account_name):
        storage_async_operation = self.client.storage_accounts.begin_create(
            resource_group_name,
            account_name,
            {
                "sku": {"name": self.sku},
                "location": self.location,
                "kind": self.kind,
                "enable_https_traffic_only": True,
            },
        )

        props = BlobServiceProperties()
        if self.is_versioning_enabled is True:
            props.is_versioning_enabled = True
        if self.container_retention_days:
            props.container_delete_retention_policy = DeleteRetentionPolicy(
                enabled=True, days=self.container_retention_days
            )
        if self.is_last_access_time_enabled:
            props.last_access_time_tracking_policy = LastAccessTimeTrackingPolicy(enable=True)

        if not all(prop is None for prop in props.as_dict().values()):
            self.client.blob_services.set_service_properties(resource_group_name, account_name, props)

        sleep(30)

        return storage_async_operation


CachedStorageAccountPreparer = functools.partial(StorageAccountPreparer, use_cache=True, random_name_enabled=True)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import argparse
import requests
import json
import logging
from github import Github, Auth
from ci_tools.functions import discover_targeted_packages

logging.getLogger().setLevel(logging.INFO)
root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "..", "..", ".."))


def get_build_info(service_directory: str, package_name: str) -> str:
    """Get the build info from the build link."""
    build_id = os.getenv("BUILD_BUILDID")
    timeline_link = f"https://dev.azure.com/azure-sdk/internal/_apis/build/builds/{build_id}/timeline?api-version=6.0"

    token = os.environ["SYSTEM_ACCESSTOKEN"]
    AUTH_HEADERS = {"Authorization": f"Bearer {token}"}

    try:
        # Make the API request
        response = requests.get(timeline_link, headers=AUTH_HEADERS)
        response_json = json.loads(response.text)

        for task in response_json["records"]:
            if "Run Pylint Next" in task["name"]:
                log_link = task["log"]["url"] + "?api-version=6.0"
                # Get the log file from the build link
                log_output = requests.get(log_link, headers=AUTH_HEADERS)
                build_output = log_output.content.decode("utf-8")
                new_output = (
                    build_output.split(
                        f"next-pylint: commands[3]> python /mnt/vss/_work/1/s/eng/tox/run_pylint.py -t {service_directory} --next=True"
                    )[1]
                ).split(f"ERROR:root:{package_name} exited with linting error")[0]
                return new_output
    except Exception as e:
        logging.error(f"Exception occurred while getting build info: {e}")
        return "Error getting build info"


def main(targeted_packages):
    for package in targeted_packages:
        # iterate through the packages and find if there is an issue associated with the package
        # there may be multiple packages in the targeted_packages list
        logging.info(f"Processing package: {package}")

        # update open GitHub issues matching the package pattern
        auth = Auth.Token(os.environ["GH_TOKEN"])
        g = Github(auth=auth)
        repo = g.get_repo("Azure/azure-sdk-for-python")

        package_name = package.split("/")[-1]
        logging.info(f"Searching for issues in repo {repo.name} for package {package_name}")

        for issue in repo.get_issues(state="open", labels=["pylint"], creator="azure-sdk"):
            if f"{package_name} needs linting updates for pylint version" in issue.title:
                logging.info(f"Found issue for package {package_name}: {issue.title}")
                build_info = get_build_info(package, package_name)

                template = (
                    f"\n\n**Pylint Errors:**"
                    f"\n\n{build_info}"
                    f"\n"
                    f"\n\n**How to fix:**"
                    f"\n**Note:** If you are a bot. Fix the issue by fixing the above pylint errors."
                )
                if issue.body and "**Pylint Errors:**" not in issue.body:
                    # If the issue body does not contain the Pylint Errors section, add it
                    first_section = issue.body.split("**How to fix:**")[0]
                    new_body = first_section + template + "\n" + issue.body.split("**How to fix:**")[1]
                    issue.edit(body=new_body)
                logging.info(f"Updated issue #{issue.number} for package {package_name}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="""
    This script is the single point for all checks invoked by CI within this repo. It works in two phases.
        1. Identify which packages in the repo are in scope for this script invocation, based on a glob string and a service directory.
        2. Invoke one or multiple `tox` environments for each package identified as in scope.

    In the case of an environment invoking `pytest`, results can be collected in a junit xml file, and test markers can be selected via --mark_arg.
    """
    )

    parser.add_argument(
        "glob_string",
        nargs="?",
        help=(
            "A comma separated list of glob strings that will target the top level directories that contain packages."
            'Examples: All = "azure-*", Single = "azure-keyvault", Targeted Multiple = "azure-keyvault,azure-mgmt-resource"'
        ),
    )

    parser.add_argument("--disablecov", help=("Flag. Disables code coverage."), action="store_true")

    parser.add_argument(
        "--service",
        help=("Name of service directory (under sdk/) to test. Example: --service applicationinsights"),
    )

    parser.add_argument(
        "-w",
        "--wheel_dir",
        dest="wheel_dir",
        help="Location for prebuilt artifacts (if any)",
    )

    parser.add_argument(
        "-i",
        "--injected-packages",
        dest="injected_packages",
        default="",
        help="Comma or space-separated list of packages that should be installed prior to dev_requirements. If local path, should be absolute.",
    )

    parser.add_argument(
        "--filter-type",
        dest="filter_type",
        default="Build",
        help="Filter type to identify eligible packages. for e.g. packages filtered in Build can pass filter type as Build,",
        choices=["Build", "Docs", "Regression", "Omit_management", "None"],
    )

    args = parser.parse_args()

    # We need to support both CI builds of everything and individual service
    # folders. This logic allows us to do both.
    if args.service and args.service != "auto":
        service_dir = os.path.join("sdk", args.service)
        target_dir = os.path.join(root_dir, service_dir)
    else:
        target_dir = root_dir

    logging.info(f"Beginning discovery for {args.service} and root dir {root_dir}. Resolving to {target_dir}.")

    if args.filter_type == "None":
        args.filter_type = "Build"
        compatibility_filter = False
    else:
        compatibility_filter = True

    targeted_packages = discover_targeted_packages(
        args.glob_string, target_dir, "", args.filter_type, compatibility_filter
    )

    if len(targeted_packages) == 0:
        logging.info(f"No packages collected for targeting string {args.glob_string} and root dir {root_dir}. Exit 0.")
        exit(0)

    main(targeted_packages)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import KubernetesVersionsOperations  # type: ignore
from ._operations import PlatformUpdatesOperations  # type: ignore
from ._operations import OsImagesOperations  # type: ignore
from ._operations import UpdateContentsOperations  # type: ignore
from ._operations import Operations  # type: ignore
from ._operations import ArcSettingsOperations  # type: ignore
from ._operations import OffersOperations  # type: ignore
from ._operations import ClustersOperations  # type: ignore
from ._operations import DeploymentSettingsOperations  # type: ignore
from ._operations import EdgeDeviceJobsOperations  # type: ignore
from ._operations import EdgeDevicesOperations  # type: ignore
from ._operations import ExtensionsOperations  # type: ignore
from ._operations import PublishersOperations  # type: ignore
from ._operations import SecuritySettingsOperations  # type: ignore
from ._operations import SkusOperations  # type: ignore
from ._operations import UpdateRunsOperations  # type: ignore
from ._operations import UpdatesOperations  # type: ignore
from ._operations import UpdateSummariesOperationGroupOperations  # type: ignore
from ._operations import ValidatedSolutionRecipesOperations  # type: ignore
from ._operations import EdgeMachinesOperations  # type: ignore
from ._operations import EdgeMachineJobsOperations  # type: ignore
from ._operations import OwnershipVouchersOperations  # type: ignore
from ._operations import ClusterJobsOperations  # type: ignore
from ._operations import DevicePoolsOperations  # type: ignore
from ._operations import UpdateSummariesOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "KubernetesVersionsOperations",
    "PlatformUpdatesOperations",
    "OsImagesOperations",
    "UpdateContentsOperations",
    "Operations",
    "ArcSettingsOperations",
    "OffersOperations",
    "ClustersOperations",
    "DeploymentSettingsOperations",
    "EdgeDeviceJobsOperations",
    "EdgeDevicesOperations",
    "ExtensionsOperations",
    "PublishersOperations",
    "SecuritySettingsOperations",
    "SkusOperations",
    "UpdateRunsOperations",
    "UpdatesOperations",
    "UpdateSummariesOperationGroupOperations",
    "ValidatedSolutionRecipesOperations",
    "EdgeMachinesOperations",
    "EdgeMachineJobsOperations",
    "OwnershipVouchersOperations",
    "ClusterJobsOperations",
    "DevicePoolsOperations",
    "UpdateSummariesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy

from . import models as _models
from ._configuration import AppPlatformManagementClientConfiguration
from ._serialization import Deserializer, Serializer
from .operations import (
    ApiPortalCustomDomainsOperations,
    ApiPortalsOperations,
    ApmsOperations,
    ApplicationAcceleratorsOperations,
    ApplicationLiveViewsOperations,
    AppsOperations,
    BindingsOperations,
    BuildServiceAgentPoolOperations,
    BuildServiceBuilderOperations,
    BuildServiceOperations,
    BuildpackBindingOperations,
    CertificatesOperations,
    ConfigServersOperations,
    ConfigurationServicesOperations,
    ContainerRegistriesOperations,
    CustomDomainsOperations,
    CustomizedAcceleratorsOperations,
    DeploymentsOperations,
    DevToolPortalsOperations,
    GatewayCustomDomainsOperations,
    GatewayRouteConfigsOperations,
    GatewaysOperations,
    MonitoringSettingsOperations,
    Operations,
    PredefinedAcceleratorsOperations,
    RuntimeVersionsOperations,
    ServiceRegistriesOperations,
    ServicesOperations,
    SkusOperations,
    StoragesOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class AppPlatformManagementClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """REST API for Azure Spring Apps.

    :ivar services: ServicesOperations operations
    :vartype services: azure.mgmt.appplatform.operations.ServicesOperations
    :ivar apms: ApmsOperations operations
    :vartype apms: azure.mgmt.appplatform.operations.ApmsOperations
    :ivar config_servers: ConfigServersOperations operations
    :vartype config_servers: azure.mgmt.appplatform.operations.ConfigServersOperations
    :ivar configuration_services: ConfigurationServicesOperations operations
    :vartype configuration_services:
     azure.mgmt.appplatform.operations.ConfigurationServicesOperations
    :ivar service_registries: ServiceRegistriesOperations operations
    :vartype service_registries: azure.mgmt.appplatform.operations.ServiceRegistriesOperations
    :ivar application_live_views: ApplicationLiveViewsOperations operations
    :vartype application_live_views:
     azure.mgmt.appplatform.operations.ApplicationLiveViewsOperations
    :ivar dev_tool_portals: DevToolPortalsOperations operations
    :vartype dev_tool_portals: azure.mgmt.appplatform.operations.DevToolPortalsOperations
    :ivar container_registries: ContainerRegistriesOperations operations
    :vartype container_registries: azure.mgmt.appplatform.operations.ContainerRegistriesOperations
    :ivar build_service: BuildServiceOperations operations
    :vartype build_service: azure.mgmt.appplatform.operations.BuildServiceOperations
    :ivar buildpack_binding: BuildpackBindingOperations operations
    :vartype buildpack_binding: azure.mgmt.appplatform.operations.BuildpackBindingOperations
    :ivar build_service_builder: BuildServiceBuilderOperations operations
    :vartype build_service_builder: azure.mgmt.appplatform.operations.BuildServiceBuilderOperations
    :ivar build_service_agent_pool: BuildServiceAgentPoolOperations operations
    :vartype build_service_agent_pool:
     azure.mgmt.appplatform.operations.BuildServiceAgentPoolOperations
    :ivar monitoring_settings: MonitoringSettingsOperations operations
    :vartype monitoring_settings: azure.mgmt.appplatform.operations.MonitoringSettingsOperations
    :ivar apps: AppsOperations operations
    :vartype apps: azure.mgmt.appplatform.operations.AppsOperations
    :ivar bindings: BindingsOperations operations
    :vartype bindings: azure.mgmt.appplatform.operations.BindingsOperations
    :ivar storages: StoragesOperations operations
    :vartype storages: azure.mgmt.appplatform.operations.StoragesOperations
    :ivar certificates: CertificatesOperations operations
    :vartype certificates: azure.mgmt.appplatform.operations.CertificatesOperations
    :ivar custom_domains: CustomDomainsOperations operations
    :vartype custom_domains: azure.mgmt.appplatform.operations.CustomDomainsOperations
    :ivar deployments: DeploymentsOperations operations
    :vartype deployments: azure.mgmt.appplatform.operations.DeploymentsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.appplatform.operations.Operations
    :ivar runtime_versions: RuntimeVersionsOperations operations
    :vartype runtime_versions: azure.mgmt.appplatform.operations.RuntimeVersionsOperations
    :ivar skus: SkusOperations operations
    :vartype skus: azure.mgmt.appplatform.operations.SkusOperations
    :ivar gateways: GatewaysOperations operations
    :vartype gateways: azure.mgmt.appplatform.operations.GatewaysOperations
    :ivar gateway_route_configs: GatewayRouteConfigsOperations operations
    :vartype gateway_route_configs: azure.mgmt.appplatform.operations.GatewayRouteConfigsOperations
    :ivar gateway_custom_domains: GatewayCustomDomainsOperations operations
    :vartype gateway_custom_domains:
     azure.mgmt.appplatform.operations.GatewayCustomDomainsOperations
    :ivar api_portals: ApiPortalsOperations operations
    :vartype api_portals: azure.mgmt.appplatform.operations.ApiPortalsOperations
    :ivar api_portal_custom_domains: ApiPortalCustomDomainsOperations operations
    :vartype api_portal_custom_domains:
     azure.mgmt.appplatform.operations.ApiPortalCustomDomainsOperations
    :ivar application_accelerators: ApplicationAcceleratorsOperations operations
    :vartype application_accelerators:
     azure.mgmt.appplatform.operations.ApplicationAcceleratorsOperations
    :ivar customized_accelerators: CustomizedAcceleratorsOperations operations
    :vartype customized_accelerators:
     azure.mgmt.appplatform.operations.CustomizedAcceleratorsOperations
    :ivar predefined_accelerators: PredefinedAcceleratorsOperations operations
    :vartype predefined_accelerators:
     azure.mgmt.appplatform.operations.PredefinedAcceleratorsOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: Gets subscription ID which uniquely identify the Microsoft Azure
     subscription. The subscription ID forms part of the URI for every service call. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2023-12-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "TokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = AppPlatformManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.services = ServicesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.apms = ApmsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.config_servers = ConfigServersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.configuration_services = ConfigurationServicesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.service_registries = ServiceRegistriesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.application_live_views = ApplicationLiveViewsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.dev_tool_portals = DevToolPortalsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.container_registries = ContainerRegistriesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.build_service = BuildServiceOperations(self._client, self._config, self._serialize, self._deserialize)
        self.buildpack_binding = BuildpackBindingOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.build_service_builder = BuildServiceBuilderOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.build_service_agent_pool = BuildServiceAgentPoolOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.monitoring_settings = MonitoringSettingsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.apps = AppsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.bindings = BindingsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.storages = StoragesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.certificates = CertificatesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.custom_domains = CustomDomainsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.deployments = DeploymentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.runtime_versions = RuntimeVersionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.skus = SkusOperations(self._client, self._config, self._serialize, self._deserialize)
        self.gateways = GatewaysOperations(self._client, self._config, self._serialize, self._deserialize)
        self.gateway_route_configs = GatewayRouteConfigsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.gateway_custom_domains = GatewayCustomDomainsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.api_portals = ApiPortalsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.api_portal_custom_domains = ApiPortalCustomDomainsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.application_accelerators = ApplicationAcceleratorsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.customized_accelerators = CustomizedAcceleratorsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.predefined_accelerators = PredefinedAcceleratorsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AFDDomain
from ._models_py3 import AFDDomainHttpsParameters
from ._models_py3 import AFDDomainListResult
from ._models_py3 import AFDDomainProperties
from ._models_py3 import AFDDomainUpdateParameters
from ._models_py3 import AFDDomainUpdatePropertiesParameters
from ._models_py3 import AFDEndpoint
from ._models_py3 import AFDEndpointListResult
from ._models_py3 import AFDEndpointProperties
from ._models_py3 import AFDEndpointPropertiesUpdateParameters
from ._models_py3 import AFDEndpointUpdateParameters
from ._models_py3 import AFDOrigin
from ._models_py3 import AFDOriginGroup
from ._models_py3 import AFDOriginGroupListResult
from ._models_py3 import AFDOriginGroupProperties
from ._models_py3 import AFDOriginGroupUpdateParameters
from ._models_py3 import AFDOriginGroupUpdatePropertiesParameters
from ._models_py3 import AFDOriginListResult
from ._models_py3 import AFDOriginProperties
from ._models_py3 import AFDOriginUpdateParameters
from ._models_py3 import AFDOriginUpdatePropertiesParameters
from ._models_py3 import AFDStateProperties
from ._models_py3 import ActivatedResourceReference
from ._models_py3 import AfdErrorResponse
from ._models_py3 import AfdPurgeParameters
from ._models_py3 import AfdRouteCacheConfiguration
from ._models_py3 import AzureFirstPartyManagedCertificate
from ._models_py3 import AzureFirstPartyManagedCertificateParameters
from ._models_py3 import CacheConfiguration
from ._models_py3 import CacheExpirationActionParameters
from ._models_py3 import CacheKeyQueryStringActionParameters
from ._models_py3 import CanMigrateParameters
from ._models_py3 import CanMigrateResult
from ._models_py3 import CdnCertificateSourceParameters
from ._models_py3 import CdnEndpoint
from ._models_py3 import CdnManagedHttpsParameters
from ._models_py3 import CdnWebApplicationFirewallPolicy
from ._models_py3 import CdnWebApplicationFirewallPolicyList
from ._models_py3 import CdnWebApplicationFirewallPolicyPatchParameters
from ._models_py3 import Certificate
from ._models_py3 import CheckEndpointNameAvailabilityInput
from ._models_py3 import CheckEndpointNameAvailabilityOutput
from ._models_py3 import CheckHostNameAvailabilityInput
from ._models_py3 import CheckNameAvailabilityInput
from ._models_py3 import CheckNameAvailabilityOutput
from ._models_py3 import CidrIpAddress
from ._models_py3 import ClientPortMatchConditionParameters
from ._models_py3 import Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems
from ._models_py3 import Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems
from ._models_py3 import ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems
from ._models_py3 import CompressionSettings
from ._models_py3 import ContinentsResponse
from ._models_py3 import ContinentsResponseContinentsItem
from ._models_py3 import ContinentsResponseCountryOrRegionsItem
from ._models_py3 import CookiesMatchConditionParameters
from ._models_py3 import CustomDomain
from ._models_py3 import CustomDomainHttpsParameters
from ._models_py3 import CustomDomainListResult
from ._models_py3 import CustomDomainParameters
from ._models_py3 import CustomRule
from ._models_py3 import CustomRuleList
from ._models_py3 import CustomerCertificate
from ._models_py3 import CustomerCertificateParameters
from ._models_py3 import DeepCreatedCustomDomain
from ._models_py3 import DeepCreatedOrigin
from ._models_py3 import DeepCreatedOriginGroup
from ._models_py3 import DeliveryRule
from ._models_py3 import DeliveryRuleAction
from ._models_py3 import DeliveryRuleCacheExpirationAction
from ._models_py3 import DeliveryRuleCacheKeyQueryStringAction
from ._models_py3 import DeliveryRuleClientPortCondition
from ._models_py3 import DeliveryRuleCondition
from ._models_py3 import DeliveryRuleCookiesCondition
from ._models_py3 import DeliveryRuleHostNameCondition
from ._models_py3 import DeliveryRuleHttpVersionCondition
from ._models_py3 import DeliveryRuleIsDeviceCondition
from ._models_py3 import DeliveryRulePostArgsCondition
from ._models_py3 import DeliveryRuleQueryStringCondition
from ._models_py3 import DeliveryRuleRemoteAddressCondition
from ._models_py3 import DeliveryRuleRequestBodyCondition
from ._models_py3 import DeliveryRuleRequestHeaderAction
from ._models_py3 import DeliveryRuleRequestHeaderCondition
from ._models_py3 import DeliveryRuleRequestMethodCondition
from ._models_py3 import DeliveryRuleRequestSchemeCondition
from ._models_py3 import DeliveryRuleRequestUriCondition
from ._models_py3 import DeliveryRuleResponseHeaderAction
from ._models_py3 import DeliveryRuleRouteConfigurationOverrideAction
from ._models_py3 import DeliveryRuleServerPortCondition
from ._models_py3 import DeliveryRuleSocketAddrCondition
from ._models_py3 import DeliveryRuleSslProtocolCondition
from ._models_py3 import DeliveryRuleUrlFileExtensionCondition
from ._models_py3 import DeliveryRuleUrlFileNameCondition
from ._models_py3 import DeliveryRuleUrlPathCondition
from ._models_py3 import DimensionProperties
from ._models_py3 import DomainValidationProperties
from ._models_py3 import EdgeNode
from ._models_py3 import EdgenodeResult
from ._models_py3 import Endpoint
from ._models_py3 import EndpointListResult
from ._models_py3 import EndpointProperties
from ._models_py3 import EndpointPropertiesUpdateParameters
from ._models_py3 import EndpointPropertiesUpdateParametersDeliveryPolicy
from ._models_py3 import EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink
from ._models_py3 import EndpointUpdateParameters
from ._models_py3 import ErrorAdditionalInfo
from ._models_py3 import ErrorDetail
from ._models_py3 import ErrorResponse
from ._models_py3 import GeoFilter
from ._models_py3 import HeaderActionParameters
from ._models_py3 import HealthProbeParameters
from ._models_py3 import HostNameMatchConditionParameters
from ._models_py3 import HttpErrorRangeParameters
from ._models_py3 import HttpVersionMatchConditionParameters
from ._models_py3 import IpAddressGroup
from ._models_py3 import IsDeviceMatchConditionParameters
from ._models_py3 import KeyVaultCertificateSourceParameters
from ._models_py3 import KeyVaultSigningKeyParameters
from ._models_py3 import LoadBalancingSettingsParameters
from ._models_py3 import LoadParameters
from ._models_py3 import LogSpecification
from ._models_py3 import ManagedCertificate
from ._models_py3 import ManagedCertificateParameters
from ._models_py3 import ManagedRuleDefinition
from ._models_py3 import ManagedRuleGroupDefinition
from ._models_py3 import ManagedRuleGroupOverride
from ._models_py3 import ManagedRuleOverride
from ._models_py3 import ManagedRuleSet
from ._models_py3 import ManagedRuleSetDefinition
from ._models_py3 import ManagedRuleSetDefinitionList
from ._models_py3 import ManagedRuleSetList
from ._models_py3 import ManagedServiceIdentity
from ._models_py3 import MatchCondition
from ._models_py3 import MetricAvailability
from ._models_py3 import MetricSpecification
from ._models_py3 import MetricsResponse
from ._models_py3 import MetricsResponseSeriesItem
from ._models_py3 import MetricsResponseSeriesPropertiesItemsItem
from ._models_py3 import MigrateResult
from ._models_py3 import MigrationErrorType
from ._models_py3 import MigrationParameters
from ._models_py3 import MigrationWebApplicationFirewallMapping
from ._models_py3 import Operation
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationsListResult
from ._models_py3 import Origin
from ._models_py3 import OriginGroup
from ._models_py3 import OriginGroupListResult
from ._models_py3 import OriginGroupOverride
from ._models_py3 import OriginGroupOverrideAction
from ._models_py3 import OriginGroupOverrideActionParameters
from ._models_py3 import OriginGroupProperties
from ._models_py3 import OriginGroupUpdateParameters
from ._models_py3 import OriginGroupUpdatePropertiesParameters
from ._models_py3 import OriginListResult
from ._models_py3 import OriginProperties
from ._models_py3 import OriginUpdateParameters
from ._models_py3 import OriginUpdatePropertiesParameters
from ._models_py3 import PolicySettings
from ._models_py3 import PostArgsMatchConditionParameters
from ._models_py3 import Profile
from ._models_py3 import ProfileChangeSkuWafMapping
from ._models_py3 import ProfileListResult
from ._models_py3 import ProfileLogScrubbing
from ._models_py3 import ProfileScrubbingRules
from ._models_py3 import ProfileUpdateParameters
from ._models_py3 import ProfileUpgradeParameters
from ._models_py3 import ProxyResource
from ._models_py3 import PurgeParameters
from ._models_py3 import QueryStringMatchConditionParameters
from ._models_py3 import RankingsResponse
from ._models_py3 import RankingsResponseTablesItem
from ._models_py3 import RankingsResponseTablesPropertiesItemsItem
from ._models_py3 import RankingsResponseTablesPropertiesItemsMetricsItem
from ._models_py3 import RateLimitRule
from ._models_py3 import RateLimitRuleList
from ._models_py3 import RemoteAddressMatchConditionParameters
from ._models_py3 import RequestBodyMatchConditionParameters
from ._models_py3 import RequestHeaderMatchConditionParameters
from ._models_py3 import RequestMethodMatchConditionParameters
from ._models_py3 import RequestSchemeMatchConditionParameters
from ._models_py3 import RequestUriMatchConditionParameters
from ._models_py3 import Resource
from ._models_py3 import ResourceReference
from ._models_py3 import ResourceUsage
from ._models_py3 import ResourceUsageListResult
from ._models_py3 import ResourcesResponse
from ._models_py3 import ResourcesResponseCustomDomainsItem
from ._models_py3 import ResourcesResponseEndpointsItem
from ._models_py3 import ResourcesResponseEndpointsPropertiesItemsItem
from ._models_py3 import ResponseBasedOriginErrorDetectionParameters
from ._models_py3 import Route
from ._models_py3 import RouteConfigurationOverrideActionParameters
from ._models_py3 import RouteListResult
from ._models_py3 import RouteProperties
from ._models_py3 import RouteUpdateParameters
from ._models_py3 import RouteUpdatePropertiesParameters
from ._models_py3 import Rule
from ._models_py3 import RuleListResult
from ._models_py3 import RuleProperties
from ._models_py3 import RuleSet
from ._models_py3 import RuleSetListResult
from ._models_py3 import RuleSetProperties
from ._models_py3 import RuleUpdateParameters
from ._models_py3 import RuleUpdatePropertiesParameters
from ._models_py3 import Secret
from ._models_py3 import SecretListResult
from ._models_py3 import SecretParameters
from ._models_py3 import SecretProperties
from ._models_py3 import SecurityPolicy
from ._models_py3 import SecurityPolicyListResult
from ._models_py3 import SecurityPolicyProperties
from ._models_py3 import SecurityPolicyPropertiesParameters
from ._models_py3 import SecurityPolicyUpdateParameters
from ._models_py3 import SecurityPolicyWebApplicationFirewallAssociation
from ._models_py3 import SecurityPolicyWebApplicationFirewallParameters
from ._models_py3 import ServerPortMatchConditionParameters
from ._models_py3 import ServiceSpecification
from ._models_py3 import SharedPrivateLinkResourceProperties
from ._models_py3 import Sku
from ._models_py3 import SocketAddrMatchConditionParameters
from ._models_py3 import SslProtocolMatchConditionParameters
from ._models_py3 import SsoUri
from ._models_py3 import SupportedOptimizationTypesListResult
from ._models_py3 import SystemData
from ._models_py3 import TrackedResource
from ._models_py3 import UrlFileExtensionMatchConditionParameters
from ._models_py3 import UrlFileNameMatchConditionParameters
from ._models_py3 import UrlPathMatchConditionParameters
from ._models_py3 import UrlRedirectAction
from ._models_py3 import UrlRedirectActionParameters
from ._models_py3 import UrlRewriteAction
from ._models_py3 import UrlRewriteActionParameters
from ._models_py3 import UrlSigningAction
from ._models_py3 import UrlSigningActionParameters
from ._models_py3 import UrlSigningKey
from ._models_py3 import UrlSigningKeyParameters
from ._models_py3 import UrlSigningParamIdentifier
from ._models_py3 import Usage
from ._models_py3 import UsageName
from ._models_py3 import UsagesListResult
from ._models_py3 import UserAssignedIdentity
from ._models_py3 import UserManagedHttpsParameters
from ._models_py3 import ValidateCustomDomainInput
from ._models_py3 import ValidateCustomDomainOutput
from ._models_py3 import ValidateProbeInput
from ._models_py3 import ValidateProbeOutput
from ._models_py3 import ValidateSecretInput
from ._models_py3 import ValidateSecretOutput
from ._models_py3 import ValidationToken
from ._models_py3 import WafMetricsResponse
from ._models_py3 import WafMetricsResponseSeriesItem
from ._models_py3 import WafMetricsResponseSeriesPropertiesItemsItem
from ._models_py3 import WafRankingsResponse
from ._models_py3 import WafRankingsResponseDataItem

from ._cdn_management_client_enums import AFDEndpointProtocols
from ._cdn_management_client_enums import ActionType
from ._cdn_management_client_enums import AfdCertificateType
from ._cdn_management_client_enums import AfdMinimumTlsVersion
from ._cdn_management_client_enums import AfdProvisioningState
from ._cdn_management_client_enums import AfdQueryStringCachingBehavior
from ._cdn_management_client_enums import Algorithm
from ._cdn_management_client_enums import AutoGeneratedDomainNameLabelScope
from ._cdn_management_client_enums import CacheBehavior
from ._cdn_management_client_enums import CacheExpirationActionParametersTypeName
from ._cdn_management_client_enums import CacheKeyQueryStringActionParametersTypeName
from ._cdn_management_client_enums import CacheType
from ._cdn_management_client_enums import CanMigrateDefaultSku
from ._cdn_management_client_enums import CdnCertificateSourceParametersTypeName
from ._cdn_management_client_enums import CertificateSource
from ._cdn_management_client_enums import CertificateType
from ._cdn_management_client_enums import ClientPortMatchConditionParametersTypeName
from ._cdn_management_client_enums import ClientPortOperator
from ._cdn_management_client_enums import CookiesMatchConditionParametersTypeName
from ._cdn_management_client_enums import CookiesOperator
from ._cdn_management_client_enums import CustomDomainResourceState
from ._cdn_management_client_enums import CustomHttpsProvisioningState
from ._cdn_management_client_enums import CustomHttpsProvisioningSubstate
from ._cdn_management_client_enums import CustomRuleEnabledState
from ._cdn_management_client_enums import DeleteRule
from ._cdn_management_client_enums import DeliveryRuleActionEnum
from ._cdn_management_client_enums import DeploymentStatus
from ._cdn_management_client_enums import DestinationProtocol
from ._cdn_management_client_enums import DomainValidationState
from ._cdn_management_client_enums import EnabledState
from ._cdn_management_client_enums import EndpointProvisioningState
from ._cdn_management_client_enums import EndpointResourceState
from ._cdn_management_client_enums import ForwardingProtocol
from ._cdn_management_client_enums import GeoFilterActions
from ._cdn_management_client_enums import HeaderAction
from ._cdn_management_client_enums import HeaderActionParametersTypeName
from ._cdn_management_client_enums import HealthProbeRequestType
from ._cdn_management_client_enums import HostNameMatchConditionParametersTypeName
from ._cdn_management_client_enums import HostNameOperator
from ._cdn_management_client_enums import HttpVersionMatchConditionParametersTypeName
from ._cdn_management_client_enums import HttpVersionOperator
from ._cdn_management_client_enums import HttpsRedirect
from ._cdn_management_client_enums import IdentityType
from ._cdn_management_client_enums import IsDeviceMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import IsDeviceMatchConditionParametersTypeName
from ._cdn_management_client_enums import IsDeviceOperator
from ._cdn_management_client_enums import KeyVaultCertificateSourceParametersTypeName
from ._cdn_management_client_enums import KeyVaultSigningKeyParametersTypeName
from ._cdn_management_client_enums import LinkToDefaultDomain
from ._cdn_management_client_enums import LogMetric
from ._cdn_management_client_enums import LogMetricsGranularity
from ._cdn_management_client_enums import LogMetricsGroupBy
from ._cdn_management_client_enums import LogRanking
from ._cdn_management_client_enums import LogRankingMetric
from ._cdn_management_client_enums import ManagedRuleEnabledState
from ._cdn_management_client_enums import ManagedServiceIdentityType
from ._cdn_management_client_enums import MatchProcessingBehavior
from ._cdn_management_client_enums import MatchVariable
from ._cdn_management_client_enums import MetricsGranularity
from ._cdn_management_client_enums import MetricsSeriesUnit
from ._cdn_management_client_enums import MinimumTlsVersion
from ._cdn_management_client_enums import Operator
from ._cdn_management_client_enums import OptimizationType
from ._cdn_management_client_enums import OriginGroupOverrideActionParametersTypeName
from ._cdn_management_client_enums import OriginGroupProvisioningState
from ._cdn_management_client_enums import OriginGroupResourceState
from ._cdn_management_client_enums import OriginProvisioningState
from ._cdn_management_client_enums import OriginResourceState
from ._cdn_management_client_enums import ParamIndicator
from ._cdn_management_client_enums import PolicyEnabledState
from ._cdn_management_client_enums import PolicyMode
from ._cdn_management_client_enums import PolicyResourceState
from ._cdn_management_client_enums import PolicySettingsDefaultCustomBlockResponseStatusCode
from ._cdn_management_client_enums import PostArgsMatchConditionParametersTypeName
from ._cdn_management_client_enums import PostArgsOperator
from ._cdn_management_client_enums import PrivateEndpointStatus
from ._cdn_management_client_enums import ProbeProtocol
from ._cdn_management_client_enums import ProfileProvisioningState
from ._cdn_management_client_enums import ProfileResourceState
from ._cdn_management_client_enums import ProfileScrubbingState
from ._cdn_management_client_enums import ProtocolType
from ._cdn_management_client_enums import ProvisioningState
from ._cdn_management_client_enums import QueryStringBehavior
from ._cdn_management_client_enums import QueryStringCachingBehavior
from ._cdn_management_client_enums import QueryStringMatchConditionParametersTypeName
from ._cdn_management_client_enums import QueryStringOperator
from ._cdn_management_client_enums import RedirectType
from ._cdn_management_client_enums import RemoteAddressMatchConditionParametersTypeName
from ._cdn_management_client_enums import RemoteAddressOperator
from ._cdn_management_client_enums import RequestBodyMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestBodyOperator
from ._cdn_management_client_enums import RequestHeaderMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestHeaderOperator
from ._cdn_management_client_enums import RequestMethodMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import RequestMethodMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestMethodOperator
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersOperator
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestUriMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestUriOperator
from ._cdn_management_client_enums import ResourceType
from ._cdn_management_client_enums import ResourceUsageUnit
from ._cdn_management_client_enums import ResponseBasedDetectedErrorTypes
from ._cdn_management_client_enums import RouteConfigurationOverrideActionParametersTypeName
from ._cdn_management_client_enums import RuleCacheBehavior
from ._cdn_management_client_enums import RuleIsCompressionEnabled
from ._cdn_management_client_enums import RuleQueryStringCachingBehavior
from ._cdn_management_client_enums import ScrubbingRuleEntryMatchOperator
from ._cdn_management_client_enums import ScrubbingRuleEntryMatchVariable
from ._cdn_management_client_enums import ScrubbingRuleEntryState
from ._cdn_management_client_enums import SecretType
from ._cdn_management_client_enums import SecurityPolicyType
from ._cdn_management_client_enums import ServerPortMatchConditionParametersTypeName
from ._cdn_management_client_enums import ServerPortOperator
from ._cdn_management_client_enums import SharedPrivateLinkResourceStatus
from ._cdn_management_client_enums import SkuName
from ._cdn_management_client_enums import SocketAddrMatchConditionParametersTypeName
from ._cdn_management_client_enums import SocketAddrOperator
from ._cdn_management_client_enums import SslProtocol
from ._cdn_management_client_enums import SslProtocolMatchConditionParametersTypeName
from ._cdn_management_client_enums import SslProtocolOperator
from ._cdn_management_client_enums import Status
from ._cdn_management_client_enums import Transform
from ._cdn_management_client_enums import TransformType
from ._cdn_management_client_enums import UpdateRule
from ._cdn_management_client_enums import UrlFileExtensionMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlFileExtensionOperator
from ._cdn_management_client_enums import UrlFileNameMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlFileNameOperator
from ._cdn_management_client_enums import UrlPathMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlPathOperator
from ._cdn_management_client_enums import UrlRedirectActionParametersTypeName
from ._cdn_management_client_enums import UrlRewriteActionParametersTypeName
from ._cdn_management_client_enums import UrlSigningActionParametersTypeName
from ._cdn_management_client_enums import UsageUnit
from ._cdn_management_client_enums import WafAction
from ._cdn_management_client_enums import WafGranularity
from ._cdn_management_client_enums import WafMatchVariable
from ._cdn_management_client_enums import WafMetric
from ._cdn_management_client_enums import WafMetricsGranularity
from ._cdn_management_client_enums import WafMetricsSeriesUnit
from ._cdn_management_client_enums import WafRankingGroupBy
from ._cdn_management_client_enums import WafRankingType
from ._cdn_management_client_enums import WafRuleType
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AFDDomain",
    "AFDDomainHttpsParameters",
    "AFDDomainListResult",
    "AFDDomainProperties",
    "AFDDomainUpdateParameters",
    "AFDDomainUpdatePropertiesParameters",
    "AFDEndpoint",
    "AFDEndpointListResult",
    "AFDEndpointProperties",
    "AFDEndpointPropertiesUpdateParameters",
    "AFDEndpointUpdateParameters",
    "AFDOrigin",
    "AFDOriginGroup",
    "AFDOriginGroupListResult",
    "AFDOriginGroupProperties",
    "AFDOriginGroupUpdateParameters",
    "AFDOriginGroupUpdatePropertiesParameters",
    "AFDOriginListResult",
    "AFDOriginProperties",
    "AFDOriginUpdateParameters",
    "AFDOriginUpdatePropertiesParameters",
    "AFDStateProperties",
    "ActivatedResourceReference",
    "AfdErrorResponse",
    "AfdPurgeParameters",
    "AfdRouteCacheConfiguration",
    "AzureFirstPartyManagedCertificate",
    "AzureFirstPartyManagedCertificateParameters",
    "CacheConfiguration",
    "CacheExpirationActionParameters",
    "CacheKeyQueryStringActionParameters",
    "CanMigrateParameters",
    "CanMigrateResult",
    "CdnCertificateSourceParameters",
    "CdnEndpoint",
    "CdnManagedHttpsParameters",
    "CdnWebApplicationFirewallPolicy",
    "CdnWebApplicationFirewallPolicyList",
    "CdnWebApplicationFirewallPolicyPatchParameters",
    "Certificate",
    "CheckEndpointNameAvailabilityInput",
    "CheckEndpointNameAvailabilityOutput",
    "CheckHostNameAvailabilityInput",
    "CheckNameAvailabilityInput",
    "CheckNameAvailabilityOutput",
    "CidrIpAddress",
    "ClientPortMatchConditionParameters",
    "Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems",
    "CompressionSettings",
    "ContinentsResponse",
    "ContinentsResponseContinentsItem",
    "ContinentsResponseCountryOrRegionsItem",
    "CookiesMatchConditionParameters",
    "CustomDomain",
    "CustomDomainHttpsParameters",
    "CustomDomainListResult",
    "CustomDomainParameters",
    "CustomRule",
    "CustomRuleList",
    "CustomerCertificate",
    "CustomerCertificateParameters",
    "DeepCreatedCustomDomain",
    "DeepCreatedOrigin",
    "DeepCreatedOriginGroup",
    "DeliveryRule",
    "DeliveryRuleAction",
    "DeliveryRuleCacheExpirationAction",
    "DeliveryRuleCacheKeyQueryStringAction",
    "DeliveryRuleClientPortCondition",
    "DeliveryRuleCondition",
    "DeliveryRuleCookiesCondition",
    "DeliveryRuleHostNameCondition",
    "DeliveryRuleHttpVersionCondition",
    "DeliveryRuleIsDeviceCondition",
    "DeliveryRulePostArgsCondition",
    "DeliveryRuleQueryStringCondition",
    "DeliveryRuleRemoteAddressCondition",
    "DeliveryRuleRequestBodyCondition",
    "DeliveryRuleRequestHeaderAction",
    "DeliveryRuleRequestHeaderCondition",
    "DeliveryRuleRequestMethodCondition",
    "DeliveryRuleRequestSchemeCondition",
    "DeliveryRuleRequestUriCondition",
    "DeliveryRuleResponseHeaderAction",
    "DeliveryRuleRouteConfigurationOverrideAction",
    "DeliveryRuleServerPortCondition",
    "DeliveryRuleSocketAddrCondition",
    "DeliveryRuleSslProtocolCondition",
    "DeliveryRuleUrlFileExtensionCondition",
    "DeliveryRuleUrlFileNameCondition",
    "DeliveryRuleUrlPathCondition",
    "DimensionProperties",
    "DomainValidationProperties",
    "EdgeNode",
    "EdgenodeResult",
    "Endpoint",
    "EndpointListResult",
    "EndpointProperties",
    "EndpointPropertiesUpdateParameters",
    "EndpointPropertiesUpdateParametersDeliveryPolicy",
    "EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink",
    "EndpointUpdateParameters",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "GeoFilter",
    "HeaderActionParameters",
    "HealthProbeParameters",
    "HostNameMatchConditionParameters",
    "HttpErrorRangeParameters",
    "HttpVersionMatchConditionParameters",
    "IpAddressGroup",
    "IsDeviceMatchConditionParameters",
    "KeyVaultCertificateSourceParameters",
    "KeyVaultSigningKeyParameters",
    "LoadBalancingSettingsParameters",
    "LoadParameters",
    "LogSpecification",
    "ManagedCertificate",
    "ManagedCertificateParameters",
    "ManagedRuleDefinition",
    "ManagedRuleGroupDefinition",
    "ManagedRuleGroupOverride",
    "ManagedRuleOverride",
    "ManagedRuleSet",
    "ManagedRuleSetDefinition",
    "ManagedRuleSetDefinitionList",
    "ManagedRuleSetList",
    "ManagedServiceIdentity",
    "MatchCondition",
    "MetricAvailability",
    "MetricSpecification",
    "MetricsResponse",
    "MetricsResponseSeriesItem",
    "MetricsResponseSeriesPropertiesItemsItem",
    "MigrateResult",
    "MigrationErrorType",
    "MigrationParameters",
    "MigrationWebApplicationFirewallMapping",
    "Operation",
    "OperationDisplay",
    "OperationsListResult",
    "Origin",
    "OriginGroup",
    "OriginGroupListResult",
    "OriginGroupOverride",
    "OriginGroupOverrideAction",
    "OriginGroupOverrideActionParameters",
    "OriginGroupProperties",
    "OriginGroupUpdateParameters",
    "OriginGroupUpdatePropertiesParameters",
    "OriginListResult",
    "OriginProperties",
    "OriginUpdateParameters",
    "OriginUpdatePropertiesParameters",
    "PolicySettings",
    "PostArgsMatchConditionParameters",
    "Profile",
    "ProfileChangeSkuWafMapping",
    "ProfileListResult",
    "ProfileLogScrubbing",
    "ProfileScrubbingRules",
    "ProfileUpdateParameters",
    "ProfileUpgradeParameters",
    "ProxyResource",
    "PurgeParameters",
    "QueryStringMatchConditionParameters",
    "RankingsResponse",
    "RankingsResponseTablesItem",
    "RankingsResponseTablesPropertiesItemsItem",
    "RankingsResponseTablesPropertiesItemsMetricsItem",
    "RateLimitRule",
    "RateLimitRuleList",
    "RemoteAddressMatchConditionParameters",
    "RequestBodyMatchConditionParameters",
    "RequestHeaderMatchConditionParameters",
    "RequestMethodMatchConditionParameters",
    "RequestSchemeMatchConditionParameters",
    "RequestUriMatchConditionParameters",
    "Resource",
    "ResourceReference",
    "ResourceUsage",
    "ResourceUsageListResult",
    "ResourcesResponse",
    "ResourcesResponseCustomDomainsItem",
    "ResourcesResponseEndpointsItem",
    "ResourcesResponseEndpointsPropertiesItemsItem",
    "ResponseBasedOriginErrorDetectionParameters",
    "Route",
    "RouteConfigurationOverrideActionParameters",
    "RouteListResult",
    "RouteProperties",
    "RouteUpdateParameters",
    "RouteUpdatePropertiesParameters",
    "Rule",
    "RuleListResult",
    "RuleProperties",
    "RuleSet",
    "RuleSetListResult",
    "RuleSetProperties",
    "RuleUpdateParameters",
    "RuleUpdatePropertiesParameters",
    "Secret",
    "SecretListResult",
    "SecretParameters",
    "SecretProperties",
    "SecurityPolicy",
    "SecurityPolicyListResult",
    "SecurityPolicyProperties",
    "SecurityPolicyPropertiesParameters",
    "SecurityPolicyUpdateParameters",
    "SecurityPolicyWebApplicationFirewallAssociation",
    "SecurityPolicyWebApplicationFirewallParameters",
    "ServerPortMatchConditionParameters",
    "ServiceSpecification",
    "SharedPrivateLinkResourceProperties",
    "Sku",
    "SocketAddrMatchConditionParameters",
    "SslProtocolMatchConditionParameters",
    "SsoUri",
    "SupportedOptimizationTypesListResult",
    "SystemData",
    "TrackedResource",
    "UrlFileExtensionMatchConditionParameters",
    "UrlFileNameMatchConditionParameters",
    "UrlPathMatchConditionParameters",
    "UrlRedirectAction",
    "UrlRedirectActionParameters",
    "UrlRewriteAction",
    "UrlRewriteActionParameters",
    "UrlSigningAction",
    "UrlSigningActionParameters",
    "UrlSigningKey",
    "UrlSigningKeyParameters",
    "UrlSigningParamIdentifier",
    "Usage",
    "UsageName",
    "UsagesListResult",
    "UserAssignedIdentity",
    "UserManagedHttpsParameters",
    "ValidateCustomDomainInput",
    "ValidateCustomDomainOutput",
    "ValidateProbeInput",
    "ValidateProbeOutput",
    "ValidateSecretInput",
    "ValidateSecretOutput",
    "ValidationToken",
    "WafMetricsResponse",
    "WafMetricsResponseSeriesItem",
    "WafMetricsResponseSeriesPropertiesItemsItem",
    "WafRankingsResponse",
    "WafRankingsResponseDataItem",
    "AFDEndpointProtocols",
    "ActionType",
    "AfdCertificateType",
    "AfdMinimumTlsVersion",
    "AfdProvisioningState",
    "AfdQueryStringCachingBehavior",
    "Algorithm",
    "AutoGeneratedDomainNameLabelScope",
    "CacheBehavior",
    "CacheExpirationActionParametersTypeName",
    "CacheKeyQueryStringActionParametersTypeName",
    "CacheType",
    "CanMigrateDefaultSku",
    "CdnCertificateSourceParametersTypeName",
    "CertificateSource",
    "CertificateType",
    "ClientPortMatchConditionParametersTypeName",
    "ClientPortOperator",
    "CookiesMatchConditionParametersTypeName",
    "CookiesOperator",
    "CustomDomainResourceState",
    "CustomHttpsProvisioningState",
    "CustomHttpsProvisioningSubstate",
    "CustomRuleEnabledState",
    "DeleteRule",
    "DeliveryRuleActionEnum",
    "DeploymentStatus",
    "DestinationProtocol",
    "DomainValidationState",
    "EnabledState",
    "EndpointProvisioningState",
    "EndpointResourceState",
    "ForwardingProtocol",
    "GeoFilterActions",
    "HeaderAction",
    "HeaderActionParametersTypeName",
    "HealthProbeRequestType",
    "HostNameMatchConditionParametersTypeName",
    "HostNameOperator",
    "HttpVersionMatchConditionParametersTypeName",
    "HttpVersionOperator",
    "HttpsRedirect",
    "IdentityType",
    "IsDeviceMatchConditionParametersMatchValuesItem",
    "IsDeviceMatchConditionParametersTypeName",
    "IsDeviceOperator",
    "KeyVaultCertificateSourceParametersTypeName",
    "KeyVaultSigningKeyParametersTypeName",
    "LinkToDefaultDomain",
    "LogMetric",
    "LogMetricsGranularity",
    "LogMetricsGroupBy",
    "LogRanking",
    "LogRankingMetric",
    "ManagedRuleEnabledState",
    "ManagedServiceIdentityType",
    "MatchProcessingBehavior",
    "MatchVariable",
    "MetricsGranularity",
    "MetricsSeriesUnit",
    "MinimumTlsVersion",
    "Operator",
    "OptimizationType",
    "OriginGroupOverrideActionParametersTypeName",
    "OriginGroupProvisioningState",
    "OriginGroupResourceState",
    "OriginProvisioningState",
    "OriginResourceState",
    "ParamIndicator",
    "PolicyEnabledState",
    "PolicyMode",
    "PolicyResourceState",
    "PolicySettingsDefaultCustomBlockResponseStatusCode",
    "PostArgsMatchConditionParametersTypeName",
    "PostArgsOperator",
    "PrivateEndpointStatus",
    "ProbeProtocol",
    "ProfileProvisioningState",
    "ProfileResourceState",
    "ProfileScrubbingState",
    "ProtocolType",
    "ProvisioningState",
    "QueryStringBehavior",
    "QueryStringCachingBehavior",
    "QueryStringMatchConditionParametersTypeName",
    "QueryStringOperator",
    "RedirectType",
    "RemoteAddressMatchConditionParametersTypeName",
    "RemoteAddressOperator",
    "RequestBodyMatchConditionParametersTypeName",
    "RequestBodyOperator",
    "RequestHeaderMatchConditionParametersTypeName",
    "RequestHeaderOperator",
    "RequestMethodMatchConditionParametersMatchValuesItem",
    "RequestMethodMatchConditionParametersTypeName",
    "RequestMethodOperator",
    "RequestSchemeMatchConditionParametersMatchValuesItem",
    "RequestSchemeMatchConditionParametersOperator",
    "RequestSchemeMatchConditionParametersTypeName",
    "RequestUriMatchConditionParametersTypeName",
    "RequestUriOperator",
    "ResourceType",
    "ResourceUsageUnit",
    "ResponseBasedDetectedErrorTypes",
    "RouteConfigurationOverrideActionParametersTypeName",
    "RuleCacheBehavior",
    "RuleIsCompressionEnabled",
    "RuleQueryStringCachingBehavior",
    "ScrubbingRuleEntryMatchOperator",
    "ScrubbingRuleEntryMatchVariable",
    "ScrubbingRuleEntryState",
    "SecretType",
    "SecurityPolicyType",
    "ServerPortMatchConditionParametersTypeName",
    "ServerPortOperator",
    "SharedPrivateLinkResourceStatus",
    "SkuName",
    "SocketAddrMatchConditionParametersTypeName",
    "SocketAddrOperator",
    "SslProtocol",
    "SslProtocolMatchConditionParametersTypeName",
    "SslProtocolOperator",
    "Status",
    "Transform",
    "TransformType",
    "UpdateRule",
    "UrlFileExtensionMatchConditionParametersTypeName",
    "UrlFileExtensionOperator",
    "UrlFileNameMatchConditionParametersTypeName",
    "UrlFileNameOperator",
    "UrlPathMatchConditionParametersTypeName",
    "UrlPathOperator",
    "UrlRedirectActionParametersTypeName",
    "UrlRewriteActionParametersTypeName",
    "UrlSigningActionParametersTypeName",
    "UsageUnit",
    "WafAction",
    "WafGranularity",
    "WafMatchVariable",
    "WafMetric",
    "WafMetricsGranularity",
    "WafMetricsSeriesUnit",
    "WafRankingGroupBy",
    "WafRankingType",
    "WafRuleType",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from azure.core import AsyncPipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ..._operations._operations import (
    build_multiple_no_operation_params_request,
    build_multiple_with_operation_path_param_request,
)
from ..._utils.utils import ClientMixinABC
from .._configuration import MultipleClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class _MultipleClientOperationsMixin(
    ClientMixinABC[AsyncPipelineClient[HttpRequest, AsyncHttpResponse], MultipleClientConfiguration]
):

    @distributed_trace_async
    async def no_operation_params(self, **kwargs: Any) -> None:
        """no_operation_params.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_multiple_no_operation_params_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "apiVersion": self._serialize.url("self._config.api_version", self._config.api_version, "str"),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace_async
    async def with_operation_path_param(self, keyword: str, **kwargs: Any) -> None:
        """with_operation_path_param.

        :param keyword: Required.
        :type keyword: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_multiple_with_operation_path_param_request(
            keyword=keyword,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "apiVersion": self._serialize.url("self._config.api_version", self._config.api_version, "str"),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""YAML-based tool Access Control List (ACL) for the Copilot adapter.

The ACL is loaded from a YAML file at startup and evaluated for every tool
permission request that the Copilot SDK emits.  Rules are checked in order;
the first matching rule's ``action`` is applied.  If no rule matches, the
``default_action`` is used (defaults to ``"deny"``).

Permission-request shape (from the Copilot SDK) by ``kind``:

``shell``
    Fields: ``fullCommandText`` (str), ``commands`` (list of
    ``{identifier: str, readOnly: bool}``), ``possiblePaths`` (list),
    ``possibleUrls`` (list), ``hasWriteFileRedirection`` (bool).

``read``
    Fields: ``path`` (str -- file path or directory being read/listed).

``write``
    Fields: ``fileName`` (str), ``diff`` (str), ``newFileContents`` (str).

``url``
    Fields: ``url`` (str -- full URL being fetched).

``mcp``
    Fields: ``toolName`` (str), ``serverName`` (str -- MCP server name).
"""
from __future__ import annotations

import logging
import os
import re
from pathlib import Path
from typing import Any, Dict, List, Literal, Optional

logger = logging.getLogger(__name__)

_Action = Literal["allow", "deny"]

# ---------------------------------------------------------------------------
# Internal data model (parsed from YAML)
# ---------------------------------------------------------------------------


class _Rule:
    """A single ACL rule."""

    __slots__ = ("kind", "action", "when")

    def __init__(
        self,
        kind: Optional[str],
        action: _Action,
        when: Dict[str, re.Pattern],
    ) -> None:
        self.kind = kind
        self.action = action
        self.when = when  # field_name -> compiled regex

    def matches(self, req: Dict[str, Any]) -> bool:
        """Return True if this rule applies to the given permission request."""
        if self.kind is not None and req.get("kind") != self.kind:
            return False
        for field, pattern in self.when.items():
            value = self._extract_field(req, field)
            if value is None:
                return False  # field absent -> rule doesn't apply
            if not pattern.search(str(value)):
                return False
        return True

    # ------------------------------------------------------------------
    # Helpers
    # ------------------------------------------------------------------

    @staticmethod
    def _extract_field(req: Dict[str, Any], field: str) -> Optional[str]:
        """Map a YAML ``when`` key to the corresponding request field."""
        if field == "command":
            return req.get("fullCommandText")
        if field == "path":
            # ``read`` uses ``path``; ``write`` uses ``fileName``
            return req.get("path") or req.get("fileName")
        if field == "url":
            return req.get("url")
        if field == "tool":
            return req.get("toolName")
        if field == "server":
            return req.get("serverName")
        # Fallback: try the field name directly
        return req.get(field)


# ---------------------------------------------------------------------------
# Public API
# ---------------------------------------------------------------------------


class ToolAcl:
    """Evaluates Copilot SDK permission requests against a YAML rule-set.

    Parameters
    ----------
    rules:
        Ordered list of :class:`_Rule` objects.
    default_action:
        Action taken when no rule matches (``"allow"`` or ``"deny"``).
        Defaults to ``"deny"`` (safe-by-default).
    source:
        Human-readable description of where the ACL was loaded from
        (used in log messages).
    """

    def __init__(
        self,
        rules: List[_Rule],
        default_action: _Action = "deny",
        source: str = "<inline>",
    ) -> None:
        self._rules = rules
        self._default = default_action
        self._source = source

    # ------------------------------------------------------------------
    # Factory
    # ------------------------------------------------------------------

    @classmethod
    def from_file(cls, path: str | os.PathLike) -> "ToolAcl":
        """Load a ``ToolAcl`` from a YAML file.

        :param path: Path to the YAML ACL file.
        :raises FileNotFoundError: If *path* does not exist.
        :raises ValueError: If the YAML is invalid or the version is unsupported.
        """
        try:
            import yaml  # type: ignore
        except ImportError as exc:
            raise ImportError(
                "PyYAML is required to load a tool ACL file.  "
                "Install it with: pip install PyYAML"
            ) from exc

        p = Path(path)
        if not p.exists():
            raise FileNotFoundError(f"Tool ACL file not found: {p}")

        with p.open(encoding="utf-8") as fh:
            data = yaml.safe_load(fh)

        return cls._parse(data, source=str(p))

    @classmethod
    def from_env(cls, env_var: str = "TOOL_ACL_PATH") -> Optional["ToolAcl"]:
        """Load a ``ToolAcl`` from the path given in an environment variable.

        Returns *None* if the variable is unset so callers can fall back to
        approve-all behaviour.

        :param env_var: Name of the environment variable (default ``TOOL_ACL_PATH``).
        """
        path = os.getenv(env_var)
        if not path:
            return None
        return cls.from_file(path)

    # ------------------------------------------------------------------
    # Evaluation
    # ------------------------------------------------------------------

    def evaluate(self, req: Dict[str, Any]) -> _Action:
        """Return ``"allow"`` or ``"deny"`` for the given permission request.

        Rules are checked in declaration order; the first matching rule's
        action is returned.  Falls back to *default_action* when nothing
        matches.
        """
        kind = req.get("kind", "?")
        text = _describe(req)
        for idx, rule in enumerate(self._rules):
            if rule.matches(req):
                logger.debug(
                    f"ACL rule #{idx + 1} ({rule.action}) matched {kind!r}: {text}"
                )
                return rule.action
        logger.debug(
            f"ACL default ({self._default}) applied to {kind!r}: {text}"
        )
        return self._default

    def is_allowed(self, req: Dict[str, Any]) -> bool:
        """Convenience wrapper: return True when :meth:`evaluate` returns ``"allow"``."""
        return self.evaluate(req) == "allow"

    # ------------------------------------------------------------------
    # Internals
    # ------------------------------------------------------------------

    @classmethod
    def _parse(cls, data: Any, source: str) -> "ToolAcl":
        if not isinstance(data, dict):
            raise ValueError(f"ACL file must be a YAML mapping, got {type(data).__name__}: {source}")

        version = str(data.get("version", "1"))
        if version != "1":
            raise ValueError(f"Unsupported ACL version {version!r} in {source}")

        raw_default = data.get("default_action", "deny")
        if raw_default not in ("allow", "deny"):
            raise ValueError(
                f"default_action must be 'allow' or 'deny', got {raw_default!r} in {source}"
            )
        default_action: _Action = raw_default  # type: ignore[assignment]

        rules: List[_Rule] = []
        for i, entry in enumerate(data.get("rules", []), start=1):
            if not isinstance(entry, dict):
                raise ValueError(f"Rule #{i} must be a mapping in {source}")
            kind = entry.get("kind")  # None means "any kind"
            raw_action = entry.get("action", "deny")
            if raw_action not in ("allow", "deny"):
                raise ValueError(
                    f"Rule #{i} action must be 'allow' or 'deny', got {raw_action!r} in {source}"
                )
            action: _Action = raw_action  # type: ignore[assignment]
            when_raw = entry.get("when", {}) or {}
            when: Dict[str, re.Pattern] = {}
            for field, pattern in when_raw.items():
                try:
                    when[field] = re.compile(pattern)
                except re.error as exc:
                    raise ValueError(
                        f"Rule #{i} when.{field} contains an invalid regex {pattern!r}: {exc}"
                    ) from exc
            rules.append(_Rule(kind=kind, action=action, when=when))

        n = len(rules)
        logger.info(
            f"Loaded tool ACL from {source}: {n} rule{'s' if n != 1 else ''}, "
            f"default={default_action!r}"
        )
        return cls(rules=rules, default_action=default_action, source=source)

    def __repr__(self) -> str:
        return (
            f"ToolAcl(rules={len(self._rules)}, default={self._default!r}, "
            f"source={self._source!r})"
        )


# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------


def _describe(req: Dict[str, Any]) -> str:
    """Return a short human-readable description of a permission request."""
    kind = req.get("kind", "?")
    if kind == "shell":
        return repr(req.get("fullCommandText", ""))
    if kind == "read":
        return repr(req.get("path", ""))
    if kind == "write":
        return repr(req.get("fileName", ""))
    if kind == "url":
        return repr(req.get("url", ""))
    if kind == "mcp":
        return f"tool={req.get('toolName', '?')!r} server={req.get('serverName', '?')!r}"
    return repr(req)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Sample 16 — Structured Outputs — Returning arbitrary structured JSON.

Return structured JSON data as a ``structured_outputs`` output item.
This sample demonstrates two approaches:

  1. **Convenience** — ``output_item_structured_outputs(data)``
  2. **Full control** — ``add_output_item_structured_outputs()`` builder

Usage::

    python sample_16_structured_outputs.py

    curl -X POST http://localhost:8088/responses \
        -H "Content-Type: application/json" \
        -d '{"model": "analysis", "input": "Analyze the product reviews"}'
"""

from azure.ai.agentserver.responses import (
    CreateResponse,
    ResponseContext,
    ResponseEventStream,
    ResponsesAgentServerHost,
)
from azure.ai.agentserver.responses.models._generated import StructuredOutputsOutputItem

app = ResponsesAgentServerHost()


# ── Variant 1: Convenience ──────────────────────────────────────────────
@app.create("structured.convenience")
async def convenience_handler(request: CreateResponse, context: ResponseContext):
    """Return structured analysis results using the convenience method."""
    stream = ResponseEventStream(response_id=context.response_id, request=request)
    yield stream.emit_created()
    yield stream.emit_in_progress()

    result = {
        "sentiment": "positive",
        "confidence": 0.95,
        "topics": ["product-quality", "customer-service"],
        "files": [
            {
                "name": "report.pdf",
                "url": "https://storage.example.com/files/report.pdf",
                "media_type": "application/pdf",
            },
        ],
    }

    async for event in stream.aoutput_item_structured_outputs(result):
        yield event

    yield stream.emit_completed()


# ── Variant 2: Full control ─────────────────────────────────────────────
@app.create("structured.full_control")
async def full_control_handler(request: CreateResponse, context: ResponseContext):
    """Return structured data using the builder for manual lifecycle control."""
    stream = ResponseEventStream(response_id=context.response_id, request=request)
    yield stream.emit_created()
    yield stream.emit_in_progress()

    builder = stream.add_output_item_structured_outputs()
    item = StructuredOutputsOutputItem(id=builder.item_id, output={"status": "ok", "count": 42})
    yield builder.emit_added(item)
    yield builder.emit_done(item)

    yield stream.emit_completed()


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app.build(), host="0.0.0.0", port=8088)
